# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
BRepMAT2d module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_brepmat2d.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _BRepMAT2d
else:
    import _BRepMAT2d

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _BRepMAT2d.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _BRepMAT2d.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepMAT2d.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _BRepMAT2d.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _BRepMAT2d.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _BRepMAT2d.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _BRepMAT2d.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _BRepMAT2d.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _BRepMAT2d.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _BRepMAT2d.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepMAT2d.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _BRepMAT2d.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _BRepMAT2d.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepMAT2d.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _BRepMAT2d.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _BRepMAT2d.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _BRepMAT2d.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _BRepMAT2d:
_BRepMAT2d.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _BRepMAT2d.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.MAT
import OCC.Core.GeomAbs
import OCC.Core.Bisector
import OCC.Core.Geom2d
import OCC.Core.gp
import OCC.Core.TColStd
import OCC.Core.TCollection
import OCC.Core.TColgp
import OCC.Core.math
import OCC.Core.Message
import OCC.Core.OSD
import OCC.Core.IntRes2d
import OCC.Core.TopoDS
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.TColGeom2d

from enum import IntEnum
from OCC.Core.Exception import *



class BRepMAT2d_DataMapOfBasicEltShape(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_begin(self)

    def end(self) -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_end(self)

    def cbegin(self) -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_cbegin(self)

    def cend(self) -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,NCollection_DefaultHasher< opencascade::handle< Standard_Transient > > >::const_iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_cend(self)

    def __init__(self, *args):
        _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_swiginit(self, _BRepMAT2d.new_BRepMAT2d_DataMapOfBasicEltShape(*args))

    def Exchange(self, theOther: "BRepMAT2d_DataMapOfBasicEltShape") -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Exchange(self, theOther)

    def Assign(self, theOther: "BRepMAT2d_DataMapOfBasicEltShape") -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,TColStd_MapTransientHasher > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Assign(self, theOther)

    def Set(self, theOther: "BRepMAT2d_DataMapOfBasicEltShape") -> "NCollection_DataMap< opencascade::handle< MAT_BasicElt >,TopoDS_Shape,TColStd_MapTransientHasher > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_ReSize(self, N)

    def Bind(self, theKey: "opencascade::handle< MAT_BasicElt > const &", theItem: "TopoDS_Shape") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Bind(self, theKey, theItem)

    def Bound(self, theKey: "opencascade::handle< MAT_BasicElt > const &", theItem: "TopoDS_Shape") -> "TopoDS_Shape *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "opencascade::handle< MAT_BasicElt > const &") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_IsBound(self, theKey)

    def UnBind(self, theKey: "opencascade::handle< MAT_BasicElt > const &") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_UnBind(self, theKey)

    def Seek(self, theKey: "opencascade::handle< MAT_BasicElt > const &") -> "TopoDS_Shape const *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Find(self, *args)

    def ChangeSeek(self, theKey: "opencascade::handle< MAT_BasicElt > const &") -> "TopoDS_Shape *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "opencascade::handle< MAT_BasicElt > const &") -> "TopoDS_Shape &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_ChangeFind(self, theKey)

    def __call__(self, *args) -> "TopoDS_Shape &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Clear(self, *args)
    __swig_destroy__ = _BRepMAT2d.delete_BRepMAT2d_DataMapOfBasicEltShape

    def Size(self) -> "Standard_Integer":
        return _BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_Size(self)

# Register BRepMAT2d_DataMapOfBasicEltShape in _BRepMAT2d:
_BRepMAT2d.BRepMAT2d_DataMapOfBasicEltShape_swigregister(BRepMAT2d_DataMapOfBasicEltShape)

class BRepMAT2d_DataMapOfShapeSequenceOfBasicElt(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_Sequence< opencascade::handle< MAT_BasicElt > >,TopTools_ShapeMapHasher >::iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_begin(self)

    def end(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_Sequence< opencascade::handle< MAT_BasicElt > >,TopTools_ShapeMapHasher >::iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_end(self)

    def cbegin(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_Sequence< opencascade::handle< MAT_BasicElt > >,TopTools_ShapeMapHasher >::const_iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_cbegin(self)

    def cend(self) -> "NCollection_DataMap< TopoDS_Shape,NCollection_Sequence< opencascade::handle< MAT_BasicElt > >,TopTools_ShapeMapHasher >::const_iterator":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_cend(self)

    def __init__(self, *args):
        _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_swiginit(self, _BRepMAT2d.new_BRepMAT2d_DataMapOfShapeSequenceOfBasicElt(*args))

    def Exchange(self, theOther: "BRepMAT2d_DataMapOfShapeSequenceOfBasicElt") -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Exchange(self, theOther)

    def Assign(self, theOther: "BRepMAT2d_DataMapOfShapeSequenceOfBasicElt") -> "NCollection_DataMap< TopoDS_Shape,MAT_SequenceOfBasicElt,TopTools_ShapeMapHasher > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Assign(self, theOther)

    def Set(self, theOther: "BRepMAT2d_DataMapOfShapeSequenceOfBasicElt") -> "NCollection_DataMap< TopoDS_Shape,MAT_SequenceOfBasicElt,TopTools_ShapeMapHasher > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Set(self, theOther)

    def ReSize(self, N: "Standard_Integer const") -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_ReSize(self, N)

    def Bind(self, theKey: "TopoDS_Shape", theItem: "MAT_SequenceOfBasicElt") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Bind(self, theKey, theItem)

    def Bound(self, theKey: "TopoDS_Shape", theItem: "MAT_SequenceOfBasicElt") -> "NCollection_Sequence< opencascade::handle< MAT_BasicElt > > *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Bound(self, theKey, theItem)

    def IsBound(self, theKey: "TopoDS_Shape") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_IsBound(self, theKey)

    def UnBind(self, theKey: "TopoDS_Shape") -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_UnBind(self, theKey)

    def Seek(self, theKey: "TopoDS_Shape") -> "NCollection_Sequence< opencascade::handle< MAT_BasicElt > > const *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Seek(self, theKey)

    def Find(self, *args) -> "Standard_Boolean":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Find(self, *args)

    def ChangeSeek(self, theKey: "TopoDS_Shape") -> "NCollection_Sequence< opencascade::handle< MAT_BasicElt > > *":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_ChangeSeek(self, theKey)

    def ChangeFind(self, theKey: "TopoDS_Shape") -> "NCollection_Sequence< opencascade::handle< MAT_BasicElt > > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_ChangeFind(self, theKey)

    def __call__(self, *args) -> "NCollection_Sequence< opencascade::handle< MAT_BasicElt > > &":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt___call__(self, *args)

    def Clear(self, *args) -> "void":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Clear(self, *args)
    __swig_destroy__ = _BRepMAT2d.delete_BRepMAT2d_DataMapOfShapeSequenceOfBasicElt

    def Size(self) -> "Standard_Integer":
        return _BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_Size(self)

# Register BRepMAT2d_DataMapOfShapeSequenceOfBasicElt in _BRepMAT2d:
_BRepMAT2d.BRepMAT2d_DataMapOfShapeSequenceOfBasicElt_swigregister(BRepMAT2d_DataMapOfShapeSequenceOfBasicElt)

class BRepMAT2d_BisectingLocus(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None

        """
        _BRepMAT2d.BRepMAT2d_BisectingLocus_swiginit(self, _BRepMAT2d.new_BRepMAT2d_BisectingLocus(*args))

    def BasicElt(self, *args) -> "opencascade::handle< MAT_BasicElt >":
        r"""
        Returns the basicelts located at the position <index> on the contour designed by <indline>. remark: the basicelts on a contour are sorted.

        Parameters
        ----------
        IndLine: int
        Index: int

        Returns
        -------
        opencascade::handle<MAT_BasicElt>

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_BasicElt(self, *args)

    def Compute(self, *args) -> "void":
        r"""
        Computation of the bisector_locus in a set of lines defined in <anexplo>. the bisecting locus are computed on the side <aside> from the line <lineindex> in <anexplo>.

        Parameters
        ----------
        anExplo: BRepMAT2d_Explorer
        LineIndex: int,optional
        	default value is 1
        aSide: MAT_Side,optional
        	default value is MAT_Left
        aJoinType: GeomAbs_JoinType,optional
        	default value is GeomAbs_Arc
        IsOpenResult: bool,optional
        	default value is Standard_False

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_Compute(self, *args)

    def GeomBis(self, *args) -> "Bisector_Bisec":
        r"""
        Returns the geometry of type <bissec> linked to the arc <arc>. <reverse> is false when the firstnode of <anarc> correspond to the first point of geometry.

        Parameters
        ----------
        anArc: MAT_Arc

        Returns
        -------
        Reverse: bool

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_GeomBis(self, *args)

    def GeomElt(self, *args) -> "gp_Pnt2d":
        r"""
        Returns the geometry linked to the <basicelt>.

        Parameters
        ----------
        aBasicElt: MAT_BasicElt

        Returns
        -------
        opencascade::handle<Geom2d_Geometry>
        Returns the geometry of type <gp> linked to the <node>.

        Parameters
        ----------
        aNode: MAT_Node

        Returns
        -------
        gp_Pnt2d

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_GeomElt(self, *args)

    def Graph(self, *args) -> "opencascade::handle< MAT_Graph >":
        r"""
        Returns <thegraph> of <self>.

        Returns
        -------
        opencascade::handle<MAT_Graph>

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_Graph(self, *args)

    def IsDone(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if compute has succeeded.

        Returns
        -------
        bool

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_IsDone(self, *args)

    def NumberOfContours(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of contours.

        Returns
        -------
        int

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_NumberOfContours(self, *args)

    def NumberOfElts(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of basicelts on the line <indline>.

        Parameters
        ----------
        IndLine: int

        Returns
        -------
        int

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_NumberOfElts(self, *args)

    def NumberOfSections(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of sections of a curve. this curve is the indexth curve in the indlineth contour given by anexplo.

        Parameters
        ----------
        IndLine: int
        Index: int

        Returns
        -------
        int

        """
        return _BRepMAT2d.BRepMAT2d_BisectingLocus_NumberOfSections(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepMAT2d.delete_BRepMAT2d_BisectingLocus

# Register BRepMAT2d_BisectingLocus in _BRepMAT2d:
_BRepMAT2d.BRepMAT2d_BisectingLocus_swigregister(BRepMAT2d_BisectingLocus)

class BRepMAT2d_Explorer(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        aFace: TopoDS_Face

        Returns
        -------
        None

        """
        _BRepMAT2d.BRepMAT2d_Explorer_swiginit(self, _BRepMAT2d.new_BRepMAT2d_Explorer(*args))

    def Clear(self, *args) -> "void":
        r"""
        Clear the contents of <self>.

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Clear(self, *args)

    def Contour(self, *args) -> "TColGeom2d_SequenceOfCurve const &":
        r"""
        No available documentation.

        Parameters
        ----------
        IndexContour: int

        Returns
        -------
        TColGeom2d_SequenceOfCurve

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Contour(self, *args)

    def GetIsClosed(self, *args) -> "TColStd_SequenceOfBoolean const &":
        r"""
        No available documentation.

        Returns
        -------
        TColStd_SequenceOfBoolean

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_GetIsClosed(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initialisation of an iterator on the curves of the contour number <indexcontour>.

        Parameters
        ----------
        IndexContour: int

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Init(self, *args)

    def IsModified(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        aShape: TopoDS_Shape

        Returns
        -------
        bool

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_IsModified(self, *args)

    def ModifiedShape(self, *args) -> "TopoDS_Shape":
        r"""
        If the shape is not modified, returns the shape itself.

        Parameters
        ----------
        aShape: TopoDS_Shape

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_ModifiedShape(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        Return false if there is no more curves on the contour initialised by the method init.

        Returns
        -------
        bool

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        Move to the next curve of the current contour.

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Next(self, *args)

    def NumberOfContours(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of contours.

        Returns
        -------
        int

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_NumberOfContours(self, *args)

    def NumberOfCurves(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of curves in the contour number <indexcontour>.

        Parameters
        ----------
        IndexContour: int

        Returns
        -------
        int

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_NumberOfCurves(self, *args)

    def Perform(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        aFace: TopoDS_Face

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Perform(self, *args)

    def Shape(self, *args) -> "TopoDS_Shape":
        r"""
        No available documentation.

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Shape(self, *args)

    def Value(self, *args) -> "opencascade::handle< Geom2d_Curve >":
        r"""
        Returns the current curve on the current contour.

        Returns
        -------
        opencascade::handle<Geom2d_Curve>

        """
        return _BRepMAT2d.BRepMAT2d_Explorer_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepMAT2d.delete_BRepMAT2d_Explorer

# Register BRepMAT2d_Explorer in _BRepMAT2d:
_BRepMAT2d.BRepMAT2d_Explorer_swigregister(BRepMAT2d_Explorer)

class BRepMAT2d_LinkTopoBilo(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        Constructs the links between s and bilo. //! raises if <s> is not a face.

        Parameters
        ----------
        Explo: BRepMAT2d_Explorer
        BiLo: BRepMAT2d_BisectingLocus

        Returns
        -------
        None

        """
        _BRepMAT2d.BRepMAT2d_LinkTopoBilo_swiginit(self, _BRepMAT2d.new_BRepMAT2d_LinkTopoBilo(*args))

    def GeneratingShape(self, *args) -> "TopoDS_Shape":
        r"""
        Returns the shape linked to <abe>.

        Parameters
        ----------
        aBE: MAT_BasicElt

        Returns
        -------
        TopoDS_Shape

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_GeneratingShape(self, *args)

    def Init(self, *args) -> "void":
        r"""
        Initialise the iterator on <s> <s> is an edge or a vertex of the initial wire or face. raises if <s> is not an edge or a vertex.

        Parameters
        ----------
        S: TopoDS_Shape

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_Init(self, *args)

    def More(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if there is a current basicelt.

        Returns
        -------
        bool

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_More(self, *args)

    def Next(self, *args) -> "void":
        r"""
        Proceed to the next basicelt.

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_Next(self, *args)

    def Perform(self, *args) -> "void":
        r"""
        Constructs the links between s and bilo. //! raises if <s> is not a face or a wire.

        Parameters
        ----------
        Explo: BRepMAT2d_Explorer
        BiLo: BRepMAT2d_BisectingLocus

        Returns
        -------
        None

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_Perform(self, *args)

    def Value(self, *args) -> "opencascade::handle< MAT_BasicElt >":
        r"""
        Returns the current basicelt.

        Returns
        -------
        opencascade::handle<MAT_BasicElt>

        """
        return _BRepMAT2d.BRepMAT2d_LinkTopoBilo_Value(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _BRepMAT2d.delete_BRepMAT2d_LinkTopoBilo

# Register BRepMAT2d_LinkTopoBilo in _BRepMAT2d:
_BRepMAT2d.BRepMAT2d_LinkTopoBilo_swigregister(BRepMAT2d_LinkTopoBilo)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Geom import *
from OCC.Core.TopoDS import *
from OCC.Core.TopTools import *
from OCC.Core.ChFiDS import *
from OCC.Core.BRepAlgo import *
from OCC.Core.TopAbs import *
from OCC.Core.GeomAbs import *
from OCC.Core.TCollection import *
from OCC.Core.BRepTools import *
from OCC.Core.TopLoc import *
from OCC.Core.Geom2d import *
from OCC.Core.gp import *


class BRepOffset_ListOfInterval:
    def __init__(self) -> None: ...
    def __len__(self) -> int: ...
    def Size(self) -> int: ...
    def Clear(self) -> None: ...
    def First(self) -> BRepOffset_Interval: ...
    def Last(self) -> BRepOffset_Interval: ...
    def Append(self, theItem: BRepOffset_Interval) -> BRepOffset_Interval: ...
    def Prepend(self, theItem: BRepOffset_Interval) -> BRepOffset_Interval: ...
    def RemoveFirst(self) -> None: ...
    def Reverse(self) -> None: ...
    def Value(self, theIndex: int) -> BRepOffset_Interval: ...
    def SetValue(self, theIndex: int, theValue: BRepOffset_Interval) -> None: ...

class BRepOffset_Error(IntEnum):
	BRepOffset_NoError: int = ...
	BRepOffset_UnknownError: int = ...
	BRepOffset_BadNormalsOnGeometry: int = ...
	BRepOffset_C0Geometry: int = ...
	BRepOffset_NullOffset: int = ...
	BRepOffset_NotConnectedShell: int = ...
	BRepOffset_CannotTrimEdges: int = ...
	BRepOffset_CannotFuseVertices: int = ...
	BRepOffset_CannotExtentEdge: int = ...
BRepOffset_NoError = BRepOffset_Error.BRepOffset_NoError
BRepOffset_UnknownError = BRepOffset_Error.BRepOffset_UnknownError
BRepOffset_BadNormalsOnGeometry = BRepOffset_Error.BRepOffset_BadNormalsOnGeometry
BRepOffset_C0Geometry = BRepOffset_Error.BRepOffset_C0Geometry
BRepOffset_NullOffset = BRepOffset_Error.BRepOffset_NullOffset
BRepOffset_NotConnectedShell = BRepOffset_Error.BRepOffset_NotConnectedShell
BRepOffset_CannotTrimEdges = BRepOffset_Error.BRepOffset_CannotTrimEdges
BRepOffset_CannotFuseVertices = BRepOffset_Error.BRepOffset_CannotFuseVertices
BRepOffset_CannotExtentEdge = BRepOffset_Error.BRepOffset_CannotExtentEdge

class BRepOffsetSimple_Status(IntEnum):
	BRepOffsetSimple_OK: int = ...
	BRepOffsetSimple_NullInputShape: int = ...
	BRepOffsetSimple_ErrorOffsetComputation: int = ...
	BRepOffsetSimple_ErrorWallFaceComputation: int = ...
	BRepOffsetSimple_ErrorInvalidNbShells: int = ...
	BRepOffsetSimple_ErrorNonClosedShell: int = ...
BRepOffsetSimple_OK = BRepOffsetSimple_Status.BRepOffsetSimple_OK
BRepOffsetSimple_NullInputShape = BRepOffsetSimple_Status.BRepOffsetSimple_NullInputShape
BRepOffsetSimple_ErrorOffsetComputation = BRepOffsetSimple_Status.BRepOffsetSimple_ErrorOffsetComputation
BRepOffsetSimple_ErrorWallFaceComputation = BRepOffsetSimple_Status.BRepOffsetSimple_ErrorWallFaceComputation
BRepOffsetSimple_ErrorInvalidNbShells = BRepOffsetSimple_Status.BRepOffsetSimple_ErrorInvalidNbShells
BRepOffsetSimple_ErrorNonClosedShell = BRepOffsetSimple_Status.BRepOffsetSimple_ErrorNonClosedShell

class BRepOffset_Mode(IntEnum):
	BRepOffset_Skin: int = ...
	BRepOffset_Pipe: int = ...
	BRepOffset_RectoVerso: int = ...
BRepOffset_Skin = BRepOffset_Mode.BRepOffset_Skin
BRepOffset_Pipe = BRepOffset_Mode.BRepOffset_Pipe
BRepOffset_RectoVerso = BRepOffset_Mode.BRepOffset_RectoVerso

class BRepOffset_Status(IntEnum):
	BRepOffset_Good: int = ...
	BRepOffset_Reversed: int = ...
	BRepOffset_Degenerated: int = ...
	BRepOffset_Unknown: int = ...
BRepOffset_Good = BRepOffset_Status.BRepOffset_Good
BRepOffset_Reversed = BRepOffset_Status.BRepOffset_Reversed
BRepOffset_Degenerated = BRepOffset_Status.BRepOffset_Degenerated
BRepOffset_Unknown = BRepOffset_Status.BRepOffset_Unknown

class brepoffset:
	@staticmethod
	def CollapseSingularities(theSurface: Geom_Surface, theFace: TopoDS_Face, thePrecision: float) -> Geom_Surface: ...
	@staticmethod
	def Surface(Surface: Geom_Surface, Offset: float, theStatus: BRepOffset_Status, allowC0: Optional[bool] = False) -> Geom_Surface: ...

class BRepOffset_Analyse:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, theS: TopoDS_Shape, theAngle: float) -> None: ...
	@overload
	def AddFaces(self, theFace: TopoDS_Face, theCo: TopoDS_Compound, theMap: TopTools_MapOfShape, theType: ChFiDS_TypeOfConcavity) -> None: ...
	@overload
	def AddFaces(self, theFace: TopoDS_Face, theCo: TopoDS_Compound, theMap: TopTools_MapOfShape, theType1: ChFiDS_TypeOfConcavity, theType2: ChFiDS_TypeOfConcavity) -> None: ...
	def Ancestors(self, theS: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def Clear(self) -> None: ...
	def Descendants(self, theS: TopoDS_Shape, theUpdate: Optional[bool] = False) -> TopTools_ListOfShape: ...
	def EdgeReplacement(self, theFace: TopoDS_Face, theEdge: TopoDS_Edge) -> TopoDS_Edge: ...
	@overload
	def Edges(self, theV: TopoDS_Vertex, theType: ChFiDS_TypeOfConcavity, theL: TopTools_ListOfShape) -> None: ...
	@overload
	def Edges(self, theF: TopoDS_Face, theType: ChFiDS_TypeOfConcavity, theL: TopTools_ListOfShape) -> None: ...
	@overload
	def Explode(self, theL: TopTools_ListOfShape, theType: ChFiDS_TypeOfConcavity) -> None: ...
	@overload
	def Explode(self, theL: TopTools_ListOfShape, theType1: ChFiDS_TypeOfConcavity, theType2: ChFiDS_TypeOfConcavity) -> None: ...
	def Generated(self, theS: TopoDS_Shape) -> TopoDS_Shape: ...
	def HasAncestor(self, theS: TopoDS_Shape) -> bool: ...
	def HasGenerated(self, theS: TopoDS_Shape) -> bool: ...
	def IsDone(self) -> bool: ...
	def NewFaces(self) -> TopTools_ListOfShape: ...
	def Perform(self, theS: TopoDS_Shape, theAngle: float) -> None: ...
	def SetFaceOffsetMap(self, theMap: TopTools_DataMapOfShapeReal) -> None: ...
	def SetOffsetValue(self, theOffset: float) -> None: ...
	def TangentEdges(self, theEdge: TopoDS_Edge, theVertex: TopoDS_Vertex, theEdges: TopTools_ListOfShape) -> None: ...
	def Type(self, theE: TopoDS_Edge) -> BRepOffset_ListOfInterval: ...

class BRepOffset_Inter2d:
	@staticmethod
	def Compute(AsDes: BRepAlgo_AsDes, F: TopoDS_Face, NewEdges: TopTools_IndexedMapOfShape, Tol: float, theDMVV: TopTools_IndexedDataMapOfShapeListOfShape) -> None: ...
	@staticmethod
	def ConnexIntByInt(FI: TopoDS_Face, OFI: BRepOffset_Offset, MES: TopTools_DataMapOfShapeShape, Build: TopTools_DataMapOfShapeShape, AsDes2d: BRepAlgo_AsDes, Offset: float, Tol: float, Analyse: BRepOffset_Analyse, FacesWithVerts: TopTools_IndexedMapOfShape, theDMVV: TopTools_IndexedDataMapOfShapeListOfShape) -> bool: ...
	@staticmethod
	def ConnexIntByIntInVert(FI: TopoDS_Face, OFI: BRepOffset_Offset, MES: TopTools_DataMapOfShapeShape, Build: TopTools_DataMapOfShapeShape, AsDes: BRepAlgo_AsDes, AsDes2d: BRepAlgo_AsDes, Tol: float, Analyse: BRepOffset_Analyse, theDMVV: TopTools_IndexedDataMapOfShapeListOfShape) -> None: ...
	@staticmethod
	def ExtentEdge(E: TopoDS_Edge, NE: TopoDS_Edge, theOffset: float) -> bool: ...
	@staticmethod
	def FuseVertices(theDMVV: TopTools_IndexedDataMapOfShapeListOfShape, theAsDes: BRepAlgo_AsDes) -> bool: ...

class BRepOffset_Inter3d:
	def __init__(self, AsDes: BRepAlgo_AsDes, Side: TopAbs_State, Tol: float) -> None: ...
	def AddCommonEdges(self, SetOfFaces: TopTools_ListOfShape) -> None: ...
	def AsDes(self) -> BRepAlgo_AsDes: ...
	def CompletInt(self, SetOfFaces: TopTools_ListOfShape, InitOffsetFace: BRepAlgo_Image) -> None: ...
	def ConnexIntByArc(self, SetOfFaces: TopTools_ListOfShape, ShapeInit: TopoDS_Shape, Analyse: BRepOffset_Analyse, InitOffsetFace: BRepAlgo_Image) -> None: ...
	def ConnexIntByInt(self, SI: TopoDS_Shape, MapSF: BRepOffset_DataMapOfShapeOffset, A: BRepOffset_Analyse, MES: TopTools_DataMapOfShapeShape, Build: TopTools_DataMapOfShapeShape, Failed: TopTools_ListOfShape, bIsPlanar: Optional[bool] = False) -> None: ...
	def ContextIntByArc(self, ContextFaces: TopTools_IndexedMapOfShape, ExtentContext: bool, Analyse: BRepOffset_Analyse, InitOffsetFace: BRepAlgo_Image, InitOffsetEdge: BRepAlgo_Image) -> None: ...
	def ContextIntByInt(self, ContextFaces: TopTools_IndexedMapOfShape, ExtentContext: bool, MapSF: BRepOffset_DataMapOfShapeOffset, A: BRepOffset_Analyse, MES: TopTools_DataMapOfShapeShape, Build: TopTools_DataMapOfShapeShape, Failed: TopTools_ListOfShape, bIsPlanar: Optional[bool] = False) -> None: ...
	def FaceInter(self, F1: TopoDS_Face, F2: TopoDS_Face, InitOffsetFace: BRepAlgo_Image) -> None: ...
	def IsDone(self, F1: TopoDS_Face, F2: TopoDS_Face) -> bool: ...
	def NewEdges(self) -> TopTools_IndexedMapOfShape: ...
	def SetDone(self, F1: TopoDS_Face, F2: TopoDS_Face) -> None: ...
	def TouchedFaces(self) -> TopTools_IndexedMapOfShape: ...

class BRepOffset_Interval:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, U1: float, U2: float, Type: ChFiDS_TypeOfConcavity) -> None: ...
	@overload
	def First(self, U: float) -> None: ...
	@overload
	def First(self) -> float: ...
	@overload
	def Last(self, U: float) -> None: ...
	@overload
	def Last(self) -> float: ...
	@overload
	def Type(self, T: ChFiDS_TypeOfConcavity) -> None: ...
	@overload
	def Type(self) -> ChFiDS_TypeOfConcavity: ...

class BRepOffset_MakeLoops:
	def __init__(self) -> None: ...
	def Build(self, LF: TopTools_ListOfShape, AsDes: BRepAlgo_AsDes, Image: BRepAlgo_Image) -> None: ...
	def BuildFaces(self, LF: TopTools_ListOfShape, AsDes: BRepAlgo_AsDes, Image: BRepAlgo_Image) -> None: ...
	def BuildOnContext(self, LContext: TopTools_ListOfShape, Analyse: BRepOffset_Analyse, AsDes: BRepAlgo_AsDes, Image: BRepAlgo_Image, InSide: bool) -> None: ...

class BRepOffset_MakeOffset:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, S: TopoDS_Shape, Offset: float, Tol: float, Mode: Optional[BRepOffset_Mode] = BRepOffset_Skin, Intersection: Optional[bool] = False, SelfInter: Optional[bool] = False, Join: Optional[GeomAbs_JoinType] = GeomAbs_Arc, Thickening: Optional[bool] = False, RemoveIntEdges: Optional[bool] = False) -> None: ...
	def AddFace(self, F: TopoDS_Face) -> None: ...
	def AllowLinearization(self, theIsAllowed: bool) -> None: ...
	def CheckInputData(self) -> bool: ...
	def Clear(self) -> None: ...
	def ClosingFaces(self) -> TopTools_IndexedMapOfShape: ...
	def Error(self) -> BRepOffset_Error: ...
	def Generated(self, theS: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def GetBadShape(self) -> TopoDS_Shape: ...
	def GetJoinType(self) -> GeomAbs_JoinType: ...
	def InitShape(self) -> TopoDS_Shape: ...
	def Initialize(self, S: TopoDS_Shape, Offset: float, Tol: float, Mode: Optional[BRepOffset_Mode] = BRepOffset_Skin, Intersection: Optional[bool] = False, SelfInter: Optional[bool] = False, Join: Optional[GeomAbs_JoinType] = GeomAbs_Arc, Thickening: Optional[bool] = False, RemoveIntEdges: Optional[bool] = False) -> None: ...
	def IsDeleted(self, S: TopoDS_Shape) -> bool: ...
	def IsDone(self) -> bool: ...
	def MakeOffsetShape(self) -> None: ...
	def MakeThickSolid(self) -> None: ...
	def Modified(self, theS: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def OffsetEdgesFromShapes(self) -> BRepAlgo_Image: ...
	def OffsetFacesFromShapes(self) -> BRepAlgo_Image: ...
	def SetOffsetOnFace(self, F: TopoDS_Face, Off: float) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...

class BRepOffset_MakeSimpleOffset:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, theInputShape: TopoDS_Shape, theOffsetValue: float) -> None: ...
	def Generated(self, theShape: TopoDS_Shape) -> TopoDS_Shape: ...
	def GetBuildSolidFlag(self) -> bool: ...
	def GetError(self) -> False: ...
	def GetErrorMessage(self) -> TCollection_AsciiString: ...
	def GetOffsetValue(self) -> float: ...
	def GetResultShape(self) -> TopoDS_Shape: ...
	def GetTolerance(self) -> float: ...
	def Initialize(self, theInputShape: TopoDS_Shape, theOffsetValue: float) -> None: ...
	def IsDone(self) -> bool: ...
	def Modified(self, theShape: TopoDS_Shape) -> TopoDS_Shape: ...
	def Perform(self) -> None: ...
	def SetBuildSolidFlag(self, theBuildFlag: bool) -> None: ...
	def SetOffsetValue(self, theOffsetValue: float) -> None: ...
	def SetTolerance(self, theValue: float) -> None: ...

class BRepOffset_Offset:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, Face: TopoDS_Face, Offset: float, OffsetOutside: Optional[bool] = True, JoinType: Optional[GeomAbs_JoinType] = GeomAbs_Arc) -> None: ...
	@overload
	def __init__(self, Face: TopoDS_Face, Offset: float, Created: TopTools_DataMapOfShapeShape, OffsetOutside: Optional[bool] = True, JoinType: Optional[GeomAbs_JoinType] = GeomAbs_Arc) -> None: ...
	@overload
	def __init__(self, Path: TopoDS_Edge, Edge1: TopoDS_Edge, Edge2: TopoDS_Edge, Offset: float, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	@overload
	def __init__(self, Path: TopoDS_Edge, Edge1: TopoDS_Edge, Edge2: TopoDS_Edge, Offset: float, FirstEdge: TopoDS_Edge, LastEdge: TopoDS_Edge, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	@overload
	def __init__(self, Vertex: TopoDS_Vertex, LEdge: TopTools_ListOfShape, Offset: float, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	def Face(self) -> TopoDS_Face: ...
	def Generated(self, Shape: TopoDS_Shape) -> TopoDS_Shape: ...
	@overload
	def Init(self, Face: TopoDS_Face, Offset: float, OffsetOutside: Optional[bool] = True, JoinType: Optional[GeomAbs_JoinType] = GeomAbs_Arc) -> None: ...
	@overload
	def Init(self, Face: TopoDS_Face, Offset: float, Created: TopTools_DataMapOfShapeShape, OffsetOutside: Optional[bool] = True, JoinType: Optional[GeomAbs_JoinType] = GeomAbs_Arc) -> None: ...
	@overload
	def Init(self, Path: TopoDS_Edge, Edge1: TopoDS_Edge, Edge2: TopoDS_Edge, Offset: float, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	@overload
	def Init(self, Path: TopoDS_Edge, Edge1: TopoDS_Edge, Edge2: TopoDS_Edge, Offset: float, FirstEdge: TopoDS_Edge, LastEdge: TopoDS_Edge, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	@overload
	def Init(self, Vertex: TopoDS_Vertex, LEdge: TopTools_ListOfShape, Offset: float, Polynomial: Optional[bool] = False, Tol: Optional[float] = 1.0e-4, Conti: Optional[GeomAbs_Shape] = GeomAbs_C1) -> None: ...
	@overload
	def Init(self, Edge: TopoDS_Edge, Offset: float) -> None: ...
	def InitialShape(self) -> TopoDS_Shape: ...
	def Status(self) -> BRepOffset_Status: ...

class BRepOffset_SimpleOffset(BRepTools_Modification):
	def __init__(self, theInputShape: TopoDS_Shape, theOffsetValue: float, theTolerance: float) -> None: ...
	def Continuity(self, E: TopoDS_Edge, F1: TopoDS_Face, F2: TopoDS_Face, NewE: TopoDS_Edge, NewF1: TopoDS_Face, NewF2: TopoDS_Face) -> GeomAbs_Shape: ...
	def NewCurve(self, E: TopoDS_Edge, C: Geom_Curve, L: TopLoc_Location) -> Tuple[bool, float]: ...
	def NewCurve2d(self, E: TopoDS_Edge, F: TopoDS_Face, NewE: TopoDS_Edge, NewF: TopoDS_Face, C: Geom2d_Curve) -> Tuple[bool, float]: ...
	def NewParameter(self, V: TopoDS_Vertex, E: TopoDS_Edge) -> Tuple[bool, float, float]: ...
	def NewPoint(self, V: TopoDS_Vertex, P: gp_Pnt) -> Tuple[bool, float]: ...
	def NewSurface(self, F: TopoDS_Face, S: Geom_Surface, L: TopLoc_Location) -> Tuple[bool, float, bool, bool]: ...

class BRepOffset_Tool:
	@staticmethod
	def BuildNeighbour(W: TopoDS_Wire, F: TopoDS_Face, NOnV1: TopTools_DataMapOfShapeShape, NOnV2: TopTools_DataMapOfShapeShape) -> None: ...
	@staticmethod
	def CheckBounds(F: TopoDS_Face, Analyse: BRepOffset_Analyse) -> Tuple[bool, bool, bool]: ...
	@staticmethod
	def CheckPlanesNormals(theFace1: TopoDS_Face, theFace2: TopoDS_Face, theTolAng: Optional[float] = 1e-8) -> bool: ...
	@staticmethod
	def CorrectOrientation(SI: TopoDS_Shape, NewEdges: TopTools_IndexedMapOfShape, AsDes: BRepAlgo_AsDes, InitOffset: BRepAlgo_Image, Offset: float) -> None: ...
	@staticmethod
	def Deboucle3D(S: TopoDS_Shape, Boundary: TopTools_MapOfShape) -> TopoDS_Shape: ...
	@staticmethod
	def EdgeVertices(E: TopoDS_Edge, V1: TopoDS_Vertex, V2: TopoDS_Vertex) -> None: ...
	@staticmethod
	def EnLargeFace(F: TopoDS_Face, NF: TopoDS_Face, ChangeGeom: bool, UpDatePCurve: Optional[bool] = False, enlargeU: Optional[bool] = True, enlargeVfirst: Optional[bool] = True, enlargeVlast: Optional[bool] = True, theExtensionMode: Optional[int] = 1, theLenBeforeUfirst: Optional[float] = -1, theLenAfterUlast: Optional[float] = -1, theLenBeforeVfirst: Optional[float] = -1, theLenAfterVlast: Optional[float] = -1) -> bool: ...
	@staticmethod
	def ExtentFace(F: TopoDS_Face, ConstShapes: TopTools_DataMapOfShapeShape, ToBuild: TopTools_DataMapOfShapeShape, Side: TopAbs_State, TolConf: float, NF: TopoDS_Face) -> None: ...
	@overload
	@staticmethod
	def FindCommonShapes(theF1: TopoDS_Face, theF2: TopoDS_Face, theLE: TopTools_ListOfShape, theLV: TopTools_ListOfShape) -> bool: ...
	@overload
	@staticmethod
	def FindCommonShapes(theS1: TopoDS_Shape, theS2: TopoDS_Shape, theType: TopAbs_ShapeEnum, theLSC: TopTools_ListOfShape) -> bool: ...
	@staticmethod
	def Gabarit(aCurve: Geom_Curve) -> float: ...
	@staticmethod
	def Inter2d(F: TopoDS_Face, E1: TopoDS_Edge, E2: TopoDS_Edge, LV: TopTools_ListOfShape, Tol: float) -> None: ...
	@staticmethod
	def Inter3D(F1: TopoDS_Face, F2: TopoDS_Face, LInt1: TopTools_ListOfShape, LInt2: TopTools_ListOfShape, Side: TopAbs_State, RefEdge: TopoDS_Edge, IsRefEdgeDefined: Optional[bool] = False) -> None: ...
	@staticmethod
	def InterOrExtent(F1: TopoDS_Face, F2: TopoDS_Face, LInt1: TopTools_ListOfShape, LInt2: TopTools_ListOfShape, Side: TopAbs_State) -> None: ...
	@staticmethod
	def MapVertexEdges(S: TopoDS_Shape, MVE: TopTools_DataMapOfShapeListOfShape) -> None: ...
	@staticmethod
	def OrientSection(E: TopoDS_Edge, F1: TopoDS_Face, F2: TopoDS_Face, O1: TopAbs_Orientation, O2: TopAbs_Orientation) -> None: ...
	@staticmethod
	def PipeInter(F1: TopoDS_Face, F2: TopoDS_Face, LInt1: TopTools_ListOfShape, LInt2: TopTools_ListOfShape, Side: TopAbs_State) -> None: ...
	@staticmethod
	def TryProject(F1: TopoDS_Face, F2: TopoDS_Face, Edges: TopTools_ListOfShape, LInt1: TopTools_ListOfShape, LInt2: TopTools_ListOfShape, Side: TopAbs_State, TolConf: float) -> bool: ...

# harray1 classes
# harray2 classes
# hsequence classes

brepoffset_CollapseSingularities = brepoffset.CollapseSingularities
brepoffset_Surface = brepoffset.Surface
BRepOffset_Inter2d_Compute = BRepOffset_Inter2d.Compute
BRepOffset_Inter2d_ConnexIntByInt = BRepOffset_Inter2d.ConnexIntByInt
BRepOffset_Inter2d_ConnexIntByIntInVert = BRepOffset_Inter2d.ConnexIntByIntInVert
BRepOffset_Inter2d_ExtentEdge = BRepOffset_Inter2d.ExtentEdge
BRepOffset_Inter2d_FuseVertices = BRepOffset_Inter2d.FuseVertices
BRepOffset_Tool_BuildNeighbour = BRepOffset_Tool.BuildNeighbour
BRepOffset_Tool_CheckBounds = BRepOffset_Tool.CheckBounds
BRepOffset_Tool_CheckPlanesNormals = BRepOffset_Tool.CheckPlanesNormals
BRepOffset_Tool_CorrectOrientation = BRepOffset_Tool.CorrectOrientation
BRepOffset_Tool_Deboucle3D = BRepOffset_Tool.Deboucle3D
BRepOffset_Tool_EdgeVertices = BRepOffset_Tool.EdgeVertices
BRepOffset_Tool_EnLargeFace = BRepOffset_Tool.EnLargeFace
BRepOffset_Tool_ExtentFace = BRepOffset_Tool.ExtentFace
BRepOffset_Tool_FindCommonShapes = BRepOffset_Tool.FindCommonShapes
BRepOffset_Tool_FindCommonShapes = BRepOffset_Tool.FindCommonShapes
BRepOffset_Tool_Gabarit = BRepOffset_Tool.Gabarit
BRepOffset_Tool_Inter2d = BRepOffset_Tool.Inter2d
BRepOffset_Tool_Inter3D = BRepOffset_Tool.Inter3D
BRepOffset_Tool_InterOrExtent = BRepOffset_Tool.InterOrExtent
BRepOffset_Tool_MapVertexEdges = BRepOffset_Tool.MapVertexEdges
BRepOffset_Tool_OrientSection = BRepOffset_Tool.OrientSection
BRepOffset_Tool_PipeInter = BRepOffset_Tool.PipeInter
BRepOffset_Tool_TryProject = BRepOffset_Tool.TryProject



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _Tesselator
else:
    import _Tesselator

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def process_exception(error: "Standard_Failure const &", method_name: "std::string", class_name: "std::string") -> "void":
    return _Tesselator.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _Tesselator.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _Tesselator.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Tesselator.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Tesselator.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _Tesselator.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _Tesselator.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _Tesselator.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _Tesselator.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _Tesselator.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _Tesselator.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Tesselator.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _Tesselator.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _Tesselator.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Tesselator.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Tesselator.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Tesselator.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _Tesselator.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _Tesselator:
_Tesselator.SwigPyIterator_swigregister(SwigPyIterator)

class vector_float(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def iterator(self) -> "swig::SwigPyIterator *":
        return _Tesselator.vector_float_iterator(self)
    def __iter__(self):
        return self.iterator()

    def __nonzero__(self) -> "bool":
        return _Tesselator.vector_float___nonzero__(self)

    def __bool__(self) -> "bool":
        return _Tesselator.vector_float___bool__(self)

    def __len__(self) -> "std::vector< float >::size_type":
        return _Tesselator.vector_float___len__(self)

    def __getslice__(self, i: "std::vector< float >::difference_type", j: "std::vector< float >::difference_type") -> "std::vector< float,std::allocator< float > > *":
        return _Tesselator.vector_float___getslice__(self, i, j)

    def __setslice__(self, *args) -> "void":
        return _Tesselator.vector_float___setslice__(self, *args)

    def __delslice__(self, i: "std::vector< float >::difference_type", j: "std::vector< float >::difference_type") -> "void":
        return _Tesselator.vector_float___delslice__(self, i, j)

    def __delitem__(self, *args) -> "void":
        return _Tesselator.vector_float___delitem__(self, *args)

    def __getitem__(self, *args) -> "std::vector< float >::value_type const &":
        return _Tesselator.vector_float___getitem__(self, *args)

    def __setitem__(self, *args) -> "void":
        return _Tesselator.vector_float___setitem__(self, *args)

    def pop(self) -> "std::vector< float >::value_type":
        return _Tesselator.vector_float_pop(self)

    def append(self, x: "std::vector< float >::value_type const &") -> "void":
        return _Tesselator.vector_float_append(self, x)

    def empty(self) -> "bool":
        return _Tesselator.vector_float_empty(self)

    def size(self) -> "std::vector< float >::size_type":
        return _Tesselator.vector_float_size(self)

    def swap(self, v: "vector_float") -> "void":
        return _Tesselator.vector_float_swap(self, v)

    def begin(self) -> "std::vector< float >::iterator":
        return _Tesselator.vector_float_begin(self)

    def end(self) -> "std::vector< float >::iterator":
        return _Tesselator.vector_float_end(self)

    def rbegin(self) -> "std::vector< float >::reverse_iterator":
        return _Tesselator.vector_float_rbegin(self)

    def rend(self) -> "std::vector< float >::reverse_iterator":
        return _Tesselator.vector_float_rend(self)

    def clear(self) -> "void":
        return _Tesselator.vector_float_clear(self)

    def get_allocator(self) -> "std::vector< float >::allocator_type":
        return _Tesselator.vector_float_get_allocator(self)

    def pop_back(self) -> "void":
        return _Tesselator.vector_float_pop_back(self)

    def erase(self, *args) -> "std::vector< float >::iterator":
        return _Tesselator.vector_float_erase(self, *args)

    def __init__(self, *args):
        _Tesselator.vector_float_swiginit(self, _Tesselator.new_vector_float(*args))

    def push_back(self, x: "std::vector< float >::value_type const &") -> "void":
        return _Tesselator.vector_float_push_back(self, x)

    def front(self) -> "std::vector< float >::value_type const &":
        return _Tesselator.vector_float_front(self)

    def back(self) -> "std::vector< float >::value_type const &":
        return _Tesselator.vector_float_back(self)

    def assign(self, n: "std::vector< float >::size_type", x: "std::vector< float >::value_type const &") -> "void":
        return _Tesselator.vector_float_assign(self, n, x)

    def resize(self, *args) -> "void":
        return _Tesselator.vector_float_resize(self, *args)

    def insert(self, *args) -> "void":
        return _Tesselator.vector_float_insert(self, *args)

    def reserve(self, n: "std::vector< float >::size_type") -> "void":
        return _Tesselator.vector_float_reserve(self, n)

    def capacity(self) -> "std::vector< float >::size_type":
        return _Tesselator.vector_float_capacity(self)
    __swig_destroy__ = _Tesselator.delete_vector_float

# Register vector_float in _Tesselator:
_Tesselator.vector_float_swigregister(vector_float)

class ShapeTesselator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, aShape: "TopoDS_Shape"):
        r"""__init__(ShapeTesselator self, TopoDS_Shape aShape) -> ShapeTesselator"""
        _Tesselator.ShapeTesselator_swiginit(self, _Tesselator.new_ShapeTesselator(aShape))
    __swig_destroy__ = _Tesselator.delete_ShapeTesselator

    def Compute(self, compute_edges: "bool"=False, mesh_quality: "float"=1.0, parallel: "bool"=False) -> "void":
        r"""Compute(ShapeTesselator self, bool compute_edges=False, float mesh_quality=1.0, bool parallel=False)"""
        return _Tesselator.ShapeTesselator_Compute(self, compute_edges, mesh_quality, parallel)

    def GetVertex(self, ivert: "int") -> "void":
        r"""GetVertex(ShapeTesselator self, int ivert)"""
        return _Tesselator.ShapeTesselator_GetVertex(self, ivert)

    def GetNormal(self, inorm: "int") -> "void":
        r"""GetNormal(ShapeTesselator self, int inorm)"""
        return _Tesselator.ShapeTesselator_GetNormal(self, inorm)

    def GetTriangleIndex(self, triangleIdx: "int") -> "void":
        r"""GetTriangleIndex(ShapeTesselator self, int triangleIdx)"""
        return _Tesselator.ShapeTesselator_GetTriangleIndex(self, triangleIdx)

    def GetEdgeVertex(self, iEdge: "int", ivert: "int") -> "void":
        r"""GetEdgeVertex(ShapeTesselator self, int iEdge, int ivert)"""
        return _Tesselator.ShapeTesselator_GetEdgeVertex(self, iEdge, ivert)

    def VerticesList(self) -> "float *":
        r"""VerticesList(ShapeTesselator self) -> float *"""
        return _Tesselator.ShapeTesselator_VerticesList(self)

    def ObjGetTriangleCount(self) -> "int":
        r"""ObjGetTriangleCount(ShapeTesselator self) -> int"""
        return _Tesselator.ShapeTesselator_ObjGetTriangleCount(self)

    def ObjGetInvalidTriangleCount(self) -> "int":
        r"""ObjGetInvalidTriangleCount(ShapeTesselator self) -> int"""
        return _Tesselator.ShapeTesselator_ObjGetInvalidTriangleCount(self)

    def ObjGetVertexCount(self) -> "int":
        r"""ObjGetVertexCount(ShapeTesselator self) -> int"""
        return _Tesselator.ShapeTesselator_ObjGetVertexCount(self)

    def ObjGetNormalCount(self) -> "int":
        r"""ObjGetNormalCount(ShapeTesselator self) -> int"""
        return _Tesselator.ShapeTesselator_ObjGetNormalCount(self)

    def ObjGetEdgeCount(self) -> "int":
        r"""ObjGetEdgeCount(ShapeTesselator self) -> int"""
        return _Tesselator.ShapeTesselator_ObjGetEdgeCount(self)

    def ObjEdgeGetVertexCount(self, iEdge: "int") -> "int":
        r"""ObjEdgeGetVertexCount(ShapeTesselator self, int iEdge) -> int"""
        return _Tesselator.ShapeTesselator_ObjEdgeGetVertexCount(self, iEdge)

    def ExportShapeToX3DTriangleSet(self) -> "std::string":
        r"""ExportShapeToX3DTriangleSet(ShapeTesselator self) -> std::string"""
        return _Tesselator.ShapeTesselator_ExportShapeToX3DTriangleSet(self)

    def ExportShapeToThreejsJSONString(self, shape_function_name: "char *") -> "std::string":
        r"""ExportShapeToThreejsJSONString(ShapeTesselator self, char * shape_function_name) -> std::string"""
        return _Tesselator.ShapeTesselator_ExportShapeToThreejsJSONString(self, shape_function_name)

    def ExportShapeToX3D(self, filename: "char *", diffR: "int"=1, diffG: "int"=0, diffB: "int"=0) -> "void":
        r"""ExportShapeToX3D(ShapeTesselator self, char * filename, int diffR=1, int diffG=0, int diffB=0)"""
        return _Tesselator.ShapeTesselator_ExportShapeToX3D(self, filename, diffR, diffG, diffB)

    def GetVerticesPositionAsTuple(self) -> "std::vector< float,std::allocator< float > >":
        r"""GetVerticesPositionAsTuple(ShapeTesselator self) -> vector_float"""
        return _Tesselator.ShapeTesselator_GetVerticesPositionAsTuple(self)

    def GetNormalsAsTuple(self) -> "std::vector< float,std::allocator< float > >":
        r"""GetNormalsAsTuple(ShapeTesselator self) -> vector_float"""
        return _Tesselator.ShapeTesselator_GetNormalsAsTuple(self)

# Register ShapeTesselator in _Tesselator:
_Tesselator.ShapeTesselator_swigregister(ShapeTesselator)






from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Message import *
from OCC.Core.BinMDF import *
from OCC.Core.TDocStd import *
from OCC.Core.TCollection import *
from OCC.Core.PCDM import *
from OCC.Core.CDM import *
from OCC.Core.Storage import *

#the following typedef cannot be wrapped as is
BinLDrivers_VectorOfDocumentSection = NewType('BinLDrivers_VectorOfDocumentSection', Any)

class BinLDrivers_Marker(IntEnum):
	BinLDrivers_ENDATTRLIST: int = ...
	BinLDrivers_ENDLABEL: int = ...
BinLDrivers_ENDATTRLIST = BinLDrivers_Marker.BinLDrivers_ENDATTRLIST
BinLDrivers_ENDLABEL = BinLDrivers_Marker.BinLDrivers_ENDLABEL

class binldrivers:
	@staticmethod
	def AttributeDrivers(MsgDrv: Message_Messenger) -> BinMDF_ADriverTable: ...
	@staticmethod
	def DefineFormat(theApp: TDocStd_Application) -> None: ...
	@staticmethod
	def Factory(theGUID: Standard_GUID) -> Standard_Transient: ...
	@staticmethod
	def StorageVersion() -> TCollection_AsciiString: ...

class BinLDrivers_DocumentRetrievalDriver(PCDM_RetrievalDriver):
	def __init__(self) -> None: ...
	def AttributeDrivers(self, theMsgDriver: Message_Messenger) -> BinMDF_ADriverTable: ...
	def CreateDocument(self) -> CDM_Document: ...
	@overload
	def Read(self, theFileName: TCollection_ExtendedString, theNewDocument: CDM_Document, theApplication: CDM_Application, theProgress: Optional[Message_ProgressRange] = Message_ProgressRange()) -> None: ...

class BinLDrivers_DocumentSection:
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, theName: TCollection_AsciiString, isPostRead: bool) -> None: ...
	def IsPostRead(self) -> bool: ...
	def Length(self) -> False: ...
	def Name(self) -> TCollection_AsciiString: ...
	def Offset(self) -> False: ...

class BinLDrivers_DocumentStorageDriver(PCDM_StorageDriver):
	def __init__(self) -> None: ...
	def AddSection(self, theName: TCollection_AsciiString, isPostRead: Optional[bool] = True) -> None: ...
	def AttributeDrivers(self, theMsgDriver: Message_Messenger) -> BinMDF_ADriverTable: ...
	@overload
	def Write(self, theDocument: CDM_Document, theFileName: TCollection_ExtendedString, theRange: Optional[Message_ProgressRange] = Message_ProgressRange()) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

binldrivers_AttributeDrivers = binldrivers.AttributeDrivers
binldrivers_DefineFormat = binldrivers.DefineFormat
binldrivers_Factory = binldrivers.Factory
binldrivers_StorageVersion = binldrivers.StorageVersion
BinLDrivers_DocumentSection_ReadTOC = BinLDrivers_DocumentSection.ReadTOC



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.BRepAdaptor import *
from OCC.Core.TopoDS import *
from OCC.Core.TopAbs import *
from OCC.Core.ChFiDS import *
from OCC.Core.GeomAbs import *
from OCC.Core.TopOpeBRepBuild import *
from OCC.Core.Geom import *
from OCC.Core.TopTools import *
from OCC.Core.Adaptor3d import *
from OCC.Core.math import *
from OCC.Core.Law import *
from OCC.Core.gp import *
from OCC.Core.GeomAdaptor import *
from OCC.Core.TopOpeBRepDS import *
from OCC.Core.Geom2d import *
from OCC.Core.TColStd import *
from OCC.Core.Bnd import *
from OCC.Core.BRepBlend import *
from OCC.Core.IntSurf import *
from OCC.Core.GeomFill import *


class ChFi3d_FilletShape(IntEnum):
	ChFi3d_Rational: int = ...
	ChFi3d_QuasiAngular: int = ...
	ChFi3d_Polynomial: int = ...
ChFi3d_Rational = ChFi3d_FilletShape.ChFi3d_Rational
ChFi3d_QuasiAngular = ChFi3d_FilletShape.ChFi3d_QuasiAngular
ChFi3d_Polynomial = ChFi3d_FilletShape.ChFi3d_Polynomial

class chfi3d:
	@staticmethod
	def ConcaveSide(S1: BRepAdaptor_Surface, S2: BRepAdaptor_Surface, E: TopoDS_Edge, Or1: TopAbs_Orientation, Or2: TopAbs_Orientation) -> int: ...
	@staticmethod
	def DefineConnectType(E: TopoDS_Edge, F1: TopoDS_Face, F2: TopoDS_Face, SinTol: float, CorrectPoint: bool) -> ChFiDS_TypeOfConcavity: ...
	@staticmethod
	def IsTangentFaces(theEdge: TopoDS_Edge, theFace1: TopoDS_Face, theFace2: TopoDS_Face, Order: Optional[GeomAbs_Shape] = GeomAbs_G1) -> bool: ...
	@overload
	@staticmethod
	def NextSide(Or1: TopAbs_Orientation, Or2: TopAbs_Orientation, OrSave1: TopAbs_Orientation, OrSave2: TopAbs_Orientation, ChoixSauv: int) -> int: ...
	@overload
	@staticmethod
	def NextSide(Or: TopAbs_Orientation, OrSave: TopAbs_Orientation, OrFace: TopAbs_Orientation) -> None: ...
	@staticmethod
	def SameSide(Or: TopAbs_Orientation, OrSave1: TopAbs_Orientation, OrSave2: TopAbs_Orientation, OrFace1: TopAbs_Orientation, OrFace2: TopAbs_Orientation) -> bool: ...

class ChFi3d_Builder:
	def Abscissa(self, IC: int, V: TopoDS_Vertex) -> float: ...
	def BadShape(self) -> TopoDS_Shape: ...
	def Builder(self) -> TopOpeBRepBuild_HBuilder: ...
	def Closed(self, IC: int) -> bool: ...
	def ClosedAndTangent(self, IC: int) -> bool: ...
	def Compute(self) -> None: ...
	def ComputedSurface(self, IC: int, IS: int) -> Geom_Surface: ...
	@overload
	def Contains(self, E: TopoDS_Edge) -> int: ...
	@overload
	def Contains(self, E: TopoDS_Edge) -> Tuple[int, int]: ...
	def FaultyContour(self, I: int) -> int: ...
	def FaultyVertex(self, IV: int) -> TopoDS_Vertex: ...
	def FirstVertex(self, IC: int) -> TopoDS_Vertex: ...
	def Generated(self, EouV: TopoDS_Shape) -> TopTools_ListOfShape: ...
	def HasResult(self) -> bool: ...
	def IsDone(self) -> bool: ...
	def LastVertex(self, IC: int) -> TopoDS_Vertex: ...
	def Length(self, IC: int) -> float: ...
	def NbComputedSurfaces(self, IC: int) -> int: ...
	def NbElements(self) -> int: ...
	def NbFaultyContours(self) -> int: ...
	def NbFaultyVertices(self) -> int: ...
	def PerformTwoCornerbyInter(self, Index: int) -> bool: ...
	def RelativeAbscissa(self, IC: int, V: TopoDS_Vertex) -> float: ...
	def Remove(self, E: TopoDS_Edge) -> None: ...
	def Reset(self) -> None: ...
	def SetContinuity(self, InternalContinuity: GeomAbs_Shape, AngularTolerance: float) -> None: ...
	def SetParams(self, Tang: float, Tesp: float, T2d: float, TApp3d: float, TolApp2d: float, Fleche: float) -> None: ...
	def Shape(self) -> TopoDS_Shape: ...
	def SplitKPart(self, Data: ChFiDS_SurfData, SetData: ChFiDS_SequenceOfSurfData, Spine: ChFiDS_Spine, Iedge: int, S1: Adaptor3d_HSurface, I1: Adaptor3d_TopolTool, S2: Adaptor3d_HSurface, I2: Adaptor3d_TopolTool) -> Tuple[bool, bool, bool]: ...
	def StripeStatus(self, IC: int) -> ChFiDS_ErrorStatus: ...
	def Value(self, I: int) -> ChFiDS_Spine: ...

class ChFi3d_SearchSing(math_FunctionWithDerivative):
	def __init__(self, C1: Geom_Curve, C2: Geom_Curve) -> None: ...
	def Derivative(self, X: float) -> Tuple[bool, float]: ...
	def Value(self, X: float) -> Tuple[bool, float]: ...
	def Values(self, X: float) -> Tuple[bool, float, float]: ...

class ChFi3d_ChBuilder(ChFi3d_Builder):
	def __init__(self, S: TopoDS_Shape, Ta: Optional[float] = 1.0e-2) -> None: ...
	@overload
	def Add(self, E: TopoDS_Edge) -> None: ...
	@overload
	def Add(self, Dis: float, E: TopoDS_Edge) -> None: ...
	@overload
	def Add(self, Dis1: float, Dis2: float, E: TopoDS_Edge, F: TopoDS_Face) -> None: ...
	def AddDA(self, Dis: float, Angle: float, E: TopoDS_Edge, F: TopoDS_Face) -> None: ...
	def Dists(self, IC: int) -> Tuple[float, float]: ...
	def GetDist(self, IC: int) -> float: ...
	def GetDistAngle(self, IC: int) -> Tuple[float, float]: ...
	def IsChamfer(self, IC: int) -> ChFiDS_ChamfMethod: ...
	def Mode(self) -> ChFiDS_ChamfMode: ...
	def NbSurf(self, IC: int) -> int: ...
	@overload
	def PerformSurf(self, Data: ChFiDS_SequenceOfSurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, MaxStep: float, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecOnS1: bool, RecOnS2: bool, Soldep: math_Vector) -> Tuple[bool, float, float, int, int]: ...
	@overload
	def PerformSurf(self, Data: ChFiDS_SequenceOfSurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, PC1: BRepAdaptor_HCurve2d, Sref1: BRepAdaptor_HSurface, PCref1: BRepAdaptor_HCurve2d, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, Or2: TopAbs_Orientation, MaxStep: float, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP: bool, RecS: bool, RecRst: bool, Soldep: math_Vector) -> Tuple[bool, float, float]: ...
	@overload
	def PerformSurf(self, Data: ChFiDS_SequenceOfSurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, Or1: TopAbs_Orientation, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, PC2: BRepAdaptor_HCurve2d, Sref2: BRepAdaptor_HSurface, PCref2: BRepAdaptor_HCurve2d, MaxStep: float, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP: bool, RecS: bool, RecRst: bool, Soldep: math_Vector) -> Tuple[bool, float, float]: ...
	@overload
	def PerformSurf(self, Data: ChFiDS_SequenceOfSurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, PC1: BRepAdaptor_HCurve2d, Sref1: BRepAdaptor_HSurface, PCref1: BRepAdaptor_HCurve2d, Or1: TopAbs_Orientation, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, PC2: BRepAdaptor_HCurve2d, Sref2: BRepAdaptor_HSurface, PCref2: BRepAdaptor_HCurve2d, Or2: TopAbs_Orientation, MaxStep: float, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP1: bool, RecRst1: bool, RecP2: bool, RecRst2: bool, Soldep: math_Vector) -> Tuple[bool, bool, float, float]: ...
	def ResetContour(self, IC: int) -> None: ...
	def Sect(self, IC: int, IS: int) -> ChFiDS_SecHArray1: ...
	def SetDist(self, Dis: float, IC: int, F: TopoDS_Face) -> None: ...
	def SetDistAngle(self, Dis: float, Angle: float, IC: int, F: TopoDS_Face) -> None: ...
	def SetDists(self, Dis1: float, Dis2: float, IC: int, F: TopoDS_Face) -> None: ...
	def SetMode(self, theMode: ChFiDS_ChamfMode) -> None: ...
	@overload
	def SimulSurf(self, Data: ChFiDS_SurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, PC1: BRepAdaptor_HCurve2d, Sref1: BRepAdaptor_HSurface, PCref1: BRepAdaptor_HCurve2d, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, Or2: TopAbs_Orientation, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP: bool, RecS: bool, RecRst: bool, Soldep: math_Vector) -> Tuple[bool, float, float]: ...
	@overload
	def SimulSurf(self, Data: ChFiDS_SurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, Or1: TopAbs_Orientation, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, PC2: BRepAdaptor_HCurve2d, Sref2: BRepAdaptor_HSurface, PCref2: BRepAdaptor_HCurve2d, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP: bool, RecS: bool, RecRst: bool, Soldep: math_Vector) -> Tuple[bool, float, float]: ...
	@overload
	def SimulSurf(self, Data: ChFiDS_SurfData, Guide: ChFiDS_HElSpine, Spine: ChFiDS_Spine, Choix: int, S1: BRepAdaptor_HSurface, I1: Adaptor3d_TopolTool, PC1: BRepAdaptor_HCurve2d, Sref1: BRepAdaptor_HSurface, PCref1: BRepAdaptor_HCurve2d, Or1: TopAbs_Orientation, S2: BRepAdaptor_HSurface, I2: Adaptor3d_TopolTool, PC2: BRepAdaptor_HCurve2d, Sref2: BRepAdaptor_HSurface, PCref2: BRepAdaptor_HCurve2d, Or2: TopAbs_Orientation, Fleche: float, TolGuide: float, Inside: bool, Appro: bool, Forward: bool, RecP1: bool, RecRst1: bool, RecP2: bool, RecRst2: bool, Soldep: math_Vector) -> Tuple[bool, bool, float, float]: ...
	def Simulate(self, IC: int) -> None: ...

class ChFi3d_FilBuilder(ChFi3d_Builder):
	def __init__(self, S: TopoDS_Shape, FShape: Optional[ChFi3d_FilletShape] = ChFi3d_Rational, Ta: Optional[float] = 1.0e-2) -> None: ...
	@overload
	def Add(self, E: TopoDS_Edge) -> None: ...
	@overload
	def Add(self, Radius: float, E: TopoDS_Edge) -> None: ...
	def GetBounds(self, IC: int, E: TopoDS_Edge) -> Tuple[bool, float, float]: ...
	def GetFilletShape(self) -> ChFi3d_FilletShape: ...
	def GetLaw(self, IC: int, E: TopoDS_Edge) -> Law_Function: ...
	@overload
	def IsConstant(self, IC: int) -> bool: ...
	@overload
	def IsConstant(self, IC: int, E: TopoDS_Edge) -> bool: ...
	def NbSurf(self, IC: int) -> int: ...
	@overload
	def Radius(self, IC: int) -> float: ...
	@overload
	def Radius(self, IC: int, E: TopoDS_Edge) -> float: ...
	def ResetContour(self, IC: int) -> None: ...
	def Sect(self, IC: int, IS: int) -> ChFiDS_SecHArray1: ...
	def SetFilletShape(self, FShape: ChFi3d_FilletShape) -> None: ...
	def SetLaw(self, IC: int, E: TopoDS_Edge, L: Law_Function) -> None: ...
	@overload
	def SetRadius(self, C: Law_Function, IC: int, IinC: int) -> None: ...
	@overload
	def SetRadius(self, Radius: float, IC: int, E: TopoDS_Edge) -> None: ...
	@overload
	def SetRadius(self, Radius: float, IC: int, V: TopoDS_Vertex) -> None: ...
	@overload
	def SetRadius(self, UandR: gp_XY, IC: int, IinC: int) -> None: ...
	def Simulate(self, IC: int) -> None: ...
	@overload
	def UnSet(self, IC: int, E: TopoDS_Edge) -> None: ...
	@overload
	def UnSet(self, IC: int, V: TopoDS_Vertex) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

chfi3d_ConcaveSide = chfi3d.ConcaveSide
chfi3d_DefineConnectType = chfi3d.DefineConnectType
chfi3d_IsTangentFaces = chfi3d.IsTangentFaces
chfi3d_NextSide = chfi3d.NextSide
chfi3d_NextSide = chfi3d.NextSide
chfi3d_SameSide = chfi3d.SameSide



##Copyright 2018 Thomas Paviot (tpaviot@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

import os

from OCC.Core.TopoDS import TopoDS_Shape
from OCC.Core.BRepMesh import BRepMesh_IncrementalMesh
from OCC.Core.StlAPI import stlapi_Read, StlAPI_Writer
from OCC.Core.BRep import BRep_Builder
from OCC.Core.gp import gp_Pnt, gp_Dir, gp_Pnt2d
from OCC.Core.Bnd import Bnd_Box2d
from OCC.Core.TopoDS import TopoDS_Compound
from OCC.Core.IGESControl import IGESControl_Reader, IGESControl_Writer
from OCC.Core.STEPControl import STEPControl_Reader, STEPControl_Writer, STEPControl_AsIs
from OCC.Core.Interface import Interface_Static_SetCVal
from OCC.Core.IFSelect import IFSelect_RetDone, IFSelect_ItemsByEntity
from OCC.Core.TDocStd import TDocStd_Document
from OCC.Core.XCAFDoc import (XCAFDoc_DocumentTool_ShapeTool,
                              XCAFDoc_DocumentTool_ColorTool)
from OCC.Core.STEPCAFControl import STEPCAFControl_Reader
from OCC.Core.TDF import TDF_LabelSequence, TDF_Label
from OCC.Core.TCollection import TCollection_ExtendedString
from OCC.Core.Quantity import Quantity_Color, Quantity_TOC_RGB
from OCC.Core.TopLoc import TopLoc_Location
from OCC.Core.BRepBuilderAPI import BRepBuilderAPI_Transform

from OCC.Extend.TopologyUtils import (discretize_edge, get_sorted_hlr_edges,
                                      list_of_shapes_to_compound)

try:
    import svgwrite
    HAVE_SVGWRITE = True
except ImportError:
    HAVE_SVGWRITE = False

##########################
# Step import and export #
##########################
def read_step_file(filename, as_compound=True, verbosity=True):
    """ read the STEP file and returns a compound
    filename: the file path
    verbosity: optional, False by default.
    as_compound: True by default. If there are more than one shape at root,
    gather all shapes into one compound. Otherwise returns a list of shapes.
    """
    if not os.path.isfile(filename):
        raise FileNotFoundError("%s not found." % filename)

    step_reader = STEPControl_Reader()
    status = step_reader.ReadFile(filename)

    if status == IFSelect_RetDone:  # check status
        if verbosity:
            failsonly = False
            step_reader.PrintCheckLoad(failsonly, IFSelect_ItemsByEntity)
            step_reader.PrintCheckTransfer(failsonly, IFSelect_ItemsByEntity)
        transfer_result = step_reader.TransferRoots()
        if not transfer_result:
            raise AssertionError("Transfer failed.")
        _nbs = step_reader.NbShapes()
        if _nbs == 0:
            raise AssertionError("No shape to transfer.")
        elif _nbs == 1:  # most cases
            return step_reader.Shape(1)
        elif _nbs > 1:
            print("Number of shapes:", _nbs)
            shps = []
            # loop over root shapes
            for k in range(1, _nbs + 1):
                new_shp = step_reader.Shape(k)
                if not new_shp.IsNull():
                    shps.append(new_shp)
            if as_compound:
                compound, result = list_of_shapes_to_compound(shps)
                if not result:
                    print("Warning: all shapes were not added to the compound")
                return compound
            else:
                print("Warning, returns a list of shapes.")
                return shps
    else:
        raise AssertionError("Error: can't read file.")
    return None


def write_step_file(a_shape, filename, application_protocol="AP203"):
    """ exports a shape to a STEP file
    a_shape: the topods_shape to export (a compound, a solid etc.)
    filename: the filename
    application protocol: "AP203" or "AP214IS" or "AP242DIS"
    """
    # a few checks
    if a_shape.IsNull():
        raise AssertionError("Shape %s is null." % a_shape)
    if application_protocol not in ["AP203", "AP214IS", "AP242DIS"]:
        raise AssertionError("application_protocol must be either AP203 or AP214IS. You passed %s." % application_protocol)
    if os.path.isfile(filename):
        print("Warning: %s file already exists and will be replaced" % filename)
    # creates and initialise the step exporter
    step_writer = STEPControl_Writer()
    Interface_Static_SetCVal("write.step.schema", application_protocol)

    # transfer shapes and write file
    step_writer.Transfer(a_shape, STEPControl_AsIs)
    status = step_writer.Write(filename)

    if not status == IFSelect_RetDone:
        raise IOError("Error while writing shape to STEP file.")
    if not os.path.isfile(filename):
        raise IOError("File %s was not saved to filesystem." % filename)


def read_step_file_with_names_colors(filename):
    """ Returns list of tuples (topods_shape, label, color)
    Use OCAF.
    """
    if not os.path.isfile(filename):
        raise FileNotFoundError("%s not found." % filename)
    # the list:
    output_shapes = {}

    # create an handle to a document
    doc = TDocStd_Document(TCollection_ExtendedString("pythonocc-doc"))

    # Get root assembly
    shape_tool = XCAFDoc_DocumentTool_ShapeTool(doc.Main())
    color_tool = XCAFDoc_DocumentTool_ColorTool(doc.Main())
    #layer_tool = XCAFDoc_DocumentTool_LayerTool(doc.Main())
    #mat_tool = XCAFDoc_DocumentTool_MaterialTool(doc.Main())

    step_reader = STEPCAFControl_Reader()
    step_reader.SetColorMode(True)
    step_reader.SetLayerMode(True)
    step_reader.SetNameMode(True)
    step_reader.SetMatMode(True)
    step_reader.SetGDTMode(True)

    status = step_reader.ReadFile(filename)
    if status == IFSelect_RetDone:
        step_reader.Transfer(doc)

    locs = []

    def _get_sub_shapes(lab, loc):
        #global cnt, lvl
        #cnt += 1
        #print("\n[%d] level %d, handling LABEL %s\n" % (cnt, lvl, _get_label_name(lab)))
        #print()
        #print(lab.DumpToString())
        #print()
        #print("Is Assembly    :", shape_tool.IsAssembly(lab))
        #print("Is Free        :", shape_tool.IsFree(lab))
        #print("Is Shape       :", shape_tool.IsShape(lab))
        #print("Is Compound    :", shape_tool.IsCompound(lab))
        #print("Is Component   :", shape_tool.IsComponent(lab))
        #print("Is SimpleShape :", shape_tool.IsSimpleShape(lab))
        #print("Is Reference   :", shape_tool.IsReference(lab))

        #users = TDF_LabelSequence()
        #users_cnt = shape_tool.GetUsers(lab, users)
        #print("Nr Users       :", users_cnt)

        l_subss = TDF_LabelSequence()
        shape_tool.GetSubShapes(lab, l_subss)
        #print("Nb subshapes   :", l_subss.Length())
        l_comps = TDF_LabelSequence()
        shape_tool.GetComponents(lab, l_comps)
        #print("Nb components  :", l_comps.Length())
        #print()
        name = lab.GetLabelName()
        print("Name :", name)

        if shape_tool.IsAssembly(lab):
            l_c = TDF_LabelSequence()
            shape_tool.GetComponents(lab, l_c)
            for i in range(l_c.Length()):
                label = l_c.Value(i+1)
                if shape_tool.IsReference(label):
                    #print("\n########  reference label :", label)
                    label_reference = TDF_Label()
                    shape_tool.GetReferredShape(label, label_reference)
                    loc = shape_tool.GetLocation(label)
                    #print("    loc          :", loc)
                    #trans = loc.Transformation()
                    #print("    tran form    :", trans.Form())
                    #rot = trans.GetRotation()
                    #print("    rotation     :", rot)
                    #print("    X            :", rot.X())
                    #print("    Y            :", rot.Y())
                    #print("    Z            :", rot.Z())
                    #print("    W            :", rot.W())
                    #tran = trans.TranslationPart()
                    #print("    translation  :", tran)
                    #print("    X            :", tran.X())
                    #print("    Y            :", tran.Y())
                    #print("    Z            :", tran.Z())

                    locs.append(loc)
                    #print(">>>>")
                    #lvl += 1
                    _get_sub_shapes(label_reference, loc)
                    #lvl -= 1
                    #print("<<<<")
                    locs.pop()

        elif shape_tool.IsSimpleShape(lab):
            #print("\n########  simpleshape label :", lab)
            shape = shape_tool.GetShape(lab)
            #print("    all ass locs   :", locs)

            loc = TopLoc_Location()
            for l in locs:
                #print("    take loc       :", l)
                loc = loc.Multiplied(l)

            #trans = loc.Transformation()
            #print("    FINAL loc    :")
            #print("    tran form    :", trans.Form())
            #rot = trans.GetRotation()
            #print("    rotation     :", rot)
            #print("    X            :", rot.X())
            #print("    Y            :", rot.Y())
            #print("    Z            :", rot.Z())
            #print("    W            :", rot.W())
            #tran = trans.TranslationPart()
            #print("    translation  :", tran)
            #print("    X            :", tran.X())
            #print("    Y            :", tran.Y())
            #print("    Z            :", tran.Z())
            c = Quantity_Color(0.5, 0.5, 0.5, Quantity_TOC_RGB)  # default color
            colorSet = False
            if (color_tool.GetInstanceColor(shape, 0, c) or
                    color_tool.GetInstanceColor(shape, 1, c) or
                    color_tool.GetInstanceColor(shape, 2, c)):
                color_tool.SetInstanceColor(shape, 0, c)
                color_tool.SetInstanceColor(shape, 1, c)
                color_tool.SetInstanceColor(shape, 2, c)
                colorSet = True
                n = c.Name(c.Red(), c.Green(), c.Blue())
                print('    instance color Name & RGB: ', c, n, c.Red(), c.Green(), c.Blue())

            if not colorSet:
                if (color_tool.GetColor(lab, 0, c) or
                        color_tool.GetColor(lab, 1, c) or
                        color_tool.GetColor(lab, 2, c)):

                    color_tool.SetInstanceColor(shape, 0, c)
                    color_tool.SetInstanceColor(shape, 1, c)
                    color_tool.SetInstanceColor(shape, 2, c)

                    n = c.Name(c.Red(), c.Green(), c.Blue())
                    print('    shape color Name & RGB: ', c, n, c.Red(), c.Green(), c.Blue())

            shape_disp = BRepBuilderAPI_Transform(shape, loc.Transformation()).Shape()
            if not shape_disp in output_shapes:
                output_shapes[shape_disp] = [lab.GetLabelName(), c]
            for i in range(l_subss.Length()):
                lab_subs = l_subss.Value(i+1)
                #print("\n########  simpleshape subshape label :", lab)
                shape_sub = shape_tool.GetShape(lab_subs)

                c = Quantity_Color(0.5, 0.5, 0.5, Quantity_TOC_RGB)  # default color
                colorSet = False
                if (color_tool.GetInstanceColor(shape_sub, 0, c) or
                        color_tool.GetInstanceColor(shape_sub, 1, c) or
                        color_tool.GetInstanceColor(shape_sub, 2, c)):
                    color_tool.SetInstanceColor(shape_sub, 0, c)
                    color_tool.SetInstanceColor(shape_sub, 1, c)
                    color_tool.SetInstanceColor(shape_sub, 2, c)
                    colorSet = True
                    n = c.Name(c.Red(), c.Green(), c.Blue())
                    print('    instance color Name & RGB: ', c, n, c.Red(), c.Green(), c.Blue())

                if not colorSet:
                    if (color_tool.GetColor(lab_subs, 0, c) or
                            color_tool.GetColor(lab_subs, 1, c) or
                            color_tool.GetColor(lab_subs, 2, c)):
                        color_tool.SetInstanceColor(shape, 0, c)
                        color_tool.SetInstanceColor(shape, 1, c)
                        color_tool.SetInstanceColor(shape, 2, c)

                        n = c.Name(c.Red(), c.Green(), c.Blue())
                        print('    shape color Name & RGB: ', c, n, c.Red(), c.Green(), c.Blue())
                shape_to_disp = BRepBuilderAPI_Transform(shape_sub, loc.Transformation()).Shape()
                # position the subshape to display
                if not shape_to_disp in output_shapes:
                    output_shapes[shape_to_disp] = [lab_subs.GetLabelName(), c]


    def _get_shapes():
        labels = TDF_LabelSequence()
        shape_tool.GetFreeShapes(labels)
        #global cnt
        #cnt += 1

        print()
        print("Number of shapes at root :", labels.Length())
        print()
        for i in range(labels.Length()):
            root_item = labels.Value(i+1)
            _get_sub_shapes(root_item, None)
    _get_shapes()
    return output_shapes


#########################
# STL import and export #
#########################
def write_stl_file(a_shape, filename, mode="ascii", linear_deflection=0.9, angular_deflection=0.5):
    """ export the shape to a STL file
    Be careful, the shape first need to be explicitely meshed using BRepMesh_IncrementalMesh
    a_shape: the topods_shape to export
    filename: the filename
    mode: optional, "ascii" by default. Can either be "binary"
    linear_deflection: optional, default to 0.001. Lower, more occurate mesh
    angular_deflection: optional, default to 0.5. Lower, more accurate_mesh
    """
    if a_shape.IsNull():
        raise AssertionError("Shape is null.")
    if mode not in ["ascii", "binary"]:
        raise AssertionError("mode should be either ascii or binary")
    if os.path.isfile(filename):
        print("Warning: %s file already exists and will be replaced" % filename)
    # first mesh the shape
    mesh = BRepMesh_IncrementalMesh(a_shape, linear_deflection, False, angular_deflection, True)
    #mesh.SetDeflection(0.05)
    mesh.Perform()
    if not mesh.IsDone():
        raise AssertionError("Mesh is not done.")

    stl_exporter = StlAPI_Writer()
    if mode == "ascii":
        stl_exporter.SetASCIIMode(True)
    else:  # binary, just set the ASCII flag to False
        stl_exporter.SetASCIIMode(False)
    stl_exporter.Write(a_shape, filename)

    if not os.path.isfile(filename):
        raise IOError("File not written to disk.")


def read_stl_file(filename):
    """ opens a stl file, reads the content, and returns a BRep topods_shape object
    """
    if not os.path.isfile(filename):
        raise FileNotFoundError("%s not found." % filename)

    the_shape = TopoDS_Shape()
    stlapi_Read(the_shape, filename)

    if the_shape.IsNull():
        raise AssertionError("Shape is null.")

    return the_shape

######################
# IGES import/export #
######################
def read_iges_file(filename, return_as_shapes=False, verbosity=False, visible_only=False):
    """ read the IGES file and returns a compound
    filename: the file path
    return_as_shapes: optional, False by default. If True returns a list of shapes,
                      else returns a single compound
    verbosity: optionl, False by default.
    """
    if not os.path.isfile(filename):
        raise FileNotFoundError("%s not found." % filename)

    iges_reader = IGESControl_Reader()
    iges_reader.SetReadVisible(visible_only)
    status = iges_reader.ReadFile(filename)

    _shapes = []

    if status == IFSelect_RetDone:  # check status
        if verbosity:
            failsonly = False
            iges_reader.PrintCheckLoad(failsonly, IFSelect_ItemsByEntity)
            iges_reader.PrintCheckTransfer(failsonly, IFSelect_ItemsByEntity)
        iges_reader.TransferRoots()
        nbr = iges_reader.NbRootsForTransfer()
        for _ in range(1, nbr+1):
            nbs = iges_reader.NbShapes()
            if nbs == 0:
                print("At least one shape in IGES cannot be transfered")
            elif nbr == 1 and nbs == 1:
                a_res_shape = iges_reader.Shape(1)
                if a_res_shape.IsNull():
                    print("At least one shape in IGES cannot be transferred")
                else:
                    _shapes.append(a_res_shape)
            else:
                for i in range(1, nbs+1):
                    a_shape = iges_reader.Shape(i)
                    if a_shape.IsNull():
                        print("At least one shape in STEP cannot be transferred")
                    else:
                        _shapes.append(a_shape)
    # if not return as shapes
    # create a compound and store all shapes
    if not return_as_shapes:
        builder = BRep_Builder()
        compound = TopoDS_Compound()
        builder.MakeCompound(compound)
        for s in _shapes:
            builder.Add(compound, s)
        _shapes = compound
    return _shapes

def write_iges_file(a_shape, filename):
    """ exports a shape to a STEP file
    a_shape: the topods_shape to export (a compound, a solid etc.)
    filename: the filename
    application protocol: "AP203" or "AP214"
    """
    # a few checks
    if a_shape.IsNull():
        raise AssertionError("Shape is null.")
    if os.path.isfile(filename):
        print("Warning: %s file already exists and will be replaced" % filename)
    # creates and initialise the step exporter
    iges_writer = IGESControl_Writer()
    iges_writer.AddShape(a_shape)
    status = iges_writer.Write(filename)

    if status != IFSelect_RetDone:
        raise AssertionError("Not done.")
    if not os.path.isfile(filename):
        raise IOError("File not written to disk.")


##############
# SVG export #
##############
def edge_to_svg_polyline(topods_edge, tol=0.1, unit="mm"):
    """ Returns a svgwrite.Path for the edge, and the 2d bounding box
    """
    unit_factor = 1  # by default

    if unit == "mm":
        unit_factor = 1
    elif unit == "m":
        unit_factor = 1e3

    points_3d = discretize_edge(topods_edge, tol)
    points_2d = []
    box2d = Bnd_Box2d()

    for point in points_3d:
        # we tak only the first 2 coordinates (x and y, leave z)
        x_p = - point[0] * unit_factor
        y_p = point[1] * unit_factor
        box2d.Add(gp_Pnt2d(x_p, y_p))
        points_2d.append((x_p, y_p))

    return svgwrite.shapes.Polyline(points_2d, fill="none"), box2d

def export_shape_to_svg(shape, filename=None,
                        width=800, height=600, margin_left=10,
                        margin_top=30, export_hidden_edges=True,
                        location=gp_Pnt(0, 0, 0), direction=gp_Dir(1, 1, 1),
                        color="black",
                        line_width="1px",
                        unit="mm"):
    """ export a single shape to an svg file and/or string.
    shape: the TopoDS_Shape to export
    filename (optional): if provided, save to an svg file
    width, height (optional): integers, specify the canva size in pixels
    margin_left, margin_top (optional): integers, in pixel
    export_hidden_edges (optional): whether or not draw hidden edges using a dashed line
    location (optional): a gp_Pnt, the lookat
    direction (optional): to set up the projector direction
    color (optional), "default to "black".
    line_width (optional, default to 1): an integer
    """
    if shape.IsNull():
        raise AssertionError("shape is Null")

    if not HAVE_SVGWRITE:
        print("svg exporter not available because the svgwrite package is not installed.")
        print("please use '$ conda install -c conda-forge svgwrite'")
        return False

    # find all edges
    visible_edges, hidden_edges = get_sorted_hlr_edges(shape, position=location, direction=direction, export_hidden_edges=export_hidden_edges)

    # compute polylines for all edges
    # we compute a global 2d bounding box as well, to be able to compute
    # the scale factor and translation vector to apply to all 2d edges so that
    # they fit the svg canva
    global_2d_bounding_box = Bnd_Box2d()

    polylines = []
    for visible_edge in visible_edges:
        visible_svg_line, visible_edge_box2d = edge_to_svg_polyline(visible_edge, 0.1, unit)
        polylines.append(visible_svg_line)
        global_2d_bounding_box.Add(visible_edge_box2d)
    if export_hidden_edges:
        for hidden_edge in hidden_edges:
            hidden_svg_line, hidden_edge_box2d = edge_to_svg_polyline(hidden_edge, 0.1, unit)
            # hidden lines are dashed style
            hidden_svg_line.dasharray([5, 5])
            polylines.append(hidden_svg_line)
            global_2d_bounding_box.Add(hidden_edge_box2d)

    # translate and scale polylines

    # first compute shape translation and scale according to size/margins
    x_min, y_min, x_max, y_max = global_2d_bounding_box.Get()
    bb2d_width = x_max - x_min
    bb2d_height = y_max - y_min

    # build the svg drawing
    dwg = svgwrite.Drawing(filename, (width, height), debug=True)
    # adjust the view box so that the lines fit then svg canvas
    dwg.viewbox(x_min - margin_left, y_min - margin_top,
                bb2d_width + 2 * margin_left, bb2d_height + 2 * margin_top)

    for polyline in polylines:
        # apply color and style
        polyline.stroke(color, width=line_width, linecap="round")
        # then adds the polyline to the svg canva
        dwg.add(polyline)

    # export to string or file according to the user choice
    if filename is not None:
        dwg.save()
        if not os.path.isfile(filename):
            raise AssertionError("svg export failed")
        print("Shape successfully exported to %s" % filename)
        return True
    return dwg.tostring()



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Message import *
from OCC.Core.TopoDS import *
from OCC.Core.Geom import *
from OCC.Core.GeomAbs import *
from OCC.Core.gp import *
from OCC.Core.TColGeom import *
from OCC.Core.TColStd import *
from OCC.Core.TopTools import *
from OCC.Core.TopAbs import *


class ShapeExtend_Status(IntEnum):
	ShapeExtend_OK: int = ...
	ShapeExtend_DONE1: int = ...
	ShapeExtend_DONE2: int = ...
	ShapeExtend_DONE3: int = ...
	ShapeExtend_DONE4: int = ...
	ShapeExtend_DONE5: int = ...
	ShapeExtend_DONE6: int = ...
	ShapeExtend_DONE7: int = ...
	ShapeExtend_DONE8: int = ...
	ShapeExtend_DONE: int = ...
	ShapeExtend_FAIL1: int = ...
	ShapeExtend_FAIL2: int = ...
	ShapeExtend_FAIL3: int = ...
	ShapeExtend_FAIL4: int = ...
	ShapeExtend_FAIL5: int = ...
	ShapeExtend_FAIL6: int = ...
	ShapeExtend_FAIL7: int = ...
	ShapeExtend_FAIL8: int = ...
	ShapeExtend_FAIL: int = ...
ShapeExtend_OK = ShapeExtend_Status.ShapeExtend_OK
ShapeExtend_DONE1 = ShapeExtend_Status.ShapeExtend_DONE1
ShapeExtend_DONE2 = ShapeExtend_Status.ShapeExtend_DONE2
ShapeExtend_DONE3 = ShapeExtend_Status.ShapeExtend_DONE3
ShapeExtend_DONE4 = ShapeExtend_Status.ShapeExtend_DONE4
ShapeExtend_DONE5 = ShapeExtend_Status.ShapeExtend_DONE5
ShapeExtend_DONE6 = ShapeExtend_Status.ShapeExtend_DONE6
ShapeExtend_DONE7 = ShapeExtend_Status.ShapeExtend_DONE7
ShapeExtend_DONE8 = ShapeExtend_Status.ShapeExtend_DONE8
ShapeExtend_DONE = ShapeExtend_Status.ShapeExtend_DONE
ShapeExtend_FAIL1 = ShapeExtend_Status.ShapeExtend_FAIL1
ShapeExtend_FAIL2 = ShapeExtend_Status.ShapeExtend_FAIL2
ShapeExtend_FAIL3 = ShapeExtend_Status.ShapeExtend_FAIL3
ShapeExtend_FAIL4 = ShapeExtend_Status.ShapeExtend_FAIL4
ShapeExtend_FAIL5 = ShapeExtend_Status.ShapeExtend_FAIL5
ShapeExtend_FAIL6 = ShapeExtend_Status.ShapeExtend_FAIL6
ShapeExtend_FAIL7 = ShapeExtend_Status.ShapeExtend_FAIL7
ShapeExtend_FAIL8 = ShapeExtend_Status.ShapeExtend_FAIL8
ShapeExtend_FAIL = ShapeExtend_Status.ShapeExtend_FAIL

class ShapeExtend_Parametrisation(IntEnum):
	ShapeExtend_Natural: int = ...
	ShapeExtend_Uniform: int = ...
	ShapeExtend_Unitary: int = ...
ShapeExtend_Natural = ShapeExtend_Parametrisation.ShapeExtend_Natural
ShapeExtend_Uniform = ShapeExtend_Parametrisation.ShapeExtend_Uniform
ShapeExtend_Unitary = ShapeExtend_Parametrisation.ShapeExtend_Unitary

class shapeextend:
	@staticmethod
	def DecodeStatus(flag: int, status: ShapeExtend_Status) -> bool: ...
	@staticmethod
	def EncodeStatus(status: ShapeExtend_Status) -> int: ...
	@staticmethod
	def Init() -> None: ...

class ShapeExtend_BasicMsgRegistrator(Standard_Transient):
	def __init__(self) -> None: ...
	@overload
	def Send(self, object: Standard_Transient, message: Message_Msg, gravity: Message_Gravity) -> None: ...
	@overload
	def Send(self, shape: TopoDS_Shape, message: Message_Msg, gravity: Message_Gravity) -> None: ...
	@overload
	def Send(self, message: Message_Msg, gravity: Message_Gravity) -> None: ...

class ShapeExtend_ComplexCurve(Geom_Curve):
	def CheckConnectivity(self, Preci: float) -> bool: ...
	def Continuity(self) -> GeomAbs_Shape: ...
	def Curve(self, index: int) -> Geom_Curve: ...
	def D0(self, U: float, P: gp_Pnt) -> None: ...
	def D1(self, U: float, P: gp_Pnt, V1: gp_Vec) -> None: ...
	def D2(self, U: float, P: gp_Pnt, V1: gp_Vec, V2: gp_Vec) -> None: ...
	def D3(self, U: float, P: gp_Pnt, V1: gp_Vec, V2: gp_Vec, V3: gp_Vec) -> None: ...
	def DN(self, U: float, N: int) -> gp_Vec: ...
	def FirstParameter(self) -> float: ...
	def GetScaleFactor(self, ind: int) -> float: ...
	def IsCN(self, N: int) -> bool: ...
	def IsClosed(self) -> bool: ...
	def IsPeriodic(self) -> bool: ...
	def LastParameter(self) -> float: ...
	def LocalToGlobal(self, index: int, Ulocal: float) -> float: ...
	def LocateParameter(self, U: float) -> Tuple[int, float]: ...
	def NbCurves(self) -> int: ...
	def ReversedParameter(self, U: float) -> float: ...
	def Transform(self, T: gp_Trsf) -> None: ...

class ShapeExtend_CompositeSurface(Geom_Surface):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, GridSurf: TColGeom_HArray2OfSurface, param: Optional[ShapeExtend_Parametrisation] = ShapeExtend_Natural) -> None: ...
	@overload
	def __init__(self, GridSurf: TColGeom_HArray2OfSurface, UJoints: TColStd_Array1OfReal, VJoints: TColStd_Array1OfReal) -> None: ...
	def Bounds(self) -> Tuple[float, float, float, float]: ...
	def CheckConnectivity(self, prec: float) -> bool: ...
	def ComputeJointValues(self, param: Optional[ShapeExtend_Parametrisation] = ShapeExtend_Natural) -> None: ...
	def Continuity(self) -> GeomAbs_Shape: ...
	def Copy(self) -> Geom_Geometry: ...
	def D0(self, U: float, V: float, P: gp_Pnt) -> None: ...
	def D1(self, U: float, V: float, P: gp_Pnt, D1U: gp_Vec, D1V: gp_Vec) -> None: ...
	def D2(self, U: float, V: float, P: gp_Pnt, D1U: gp_Vec, D1V: gp_Vec, D2U: gp_Vec, D2V: gp_Vec, D2UV: gp_Vec) -> None: ...
	def D3(self, U: float, V: float, P: gp_Pnt, D1U: gp_Vec, D1V: gp_Vec, D2U: gp_Vec, D2V: gp_Vec, D2UV: gp_Vec, D3U: gp_Vec, D3V: gp_Vec, D3UUV: gp_Vec, D3UVV: gp_Vec) -> None: ...
	def DN(self, U: float, V: float, Nu: int, Nv: int) -> gp_Vec: ...
	def GlobalToLocal(self, i: int, j: int, UV: gp_Pnt2d) -> gp_Pnt2d: ...
	def GlobalToLocalTransformation(self, i: int, j: int, Trsf: gp_Trsf2d) -> Tuple[bool, float]: ...
	@overload
	def Init(self, GridSurf: TColGeom_HArray2OfSurface, param: Optional[ShapeExtend_Parametrisation] = ShapeExtend_Natural) -> bool: ...
	@overload
	def Init(self, GridSurf: TColGeom_HArray2OfSurface, UJoints: TColStd_Array1OfReal, VJoints: TColStd_Array1OfReal) -> bool: ...
	def IsCNu(self, N: int) -> bool: ...
	def IsCNv(self, N: int) -> bool: ...
	def IsUClosed(self) -> bool: ...
	def IsUPeriodic(self) -> bool: ...
	def IsVClosed(self) -> bool: ...
	def IsVPeriodic(self) -> bool: ...
	def LocalToGlobal(self, i: int, j: int, uv: gp_Pnt2d) -> gp_Pnt2d: ...
	def LocateUParameter(self, U: float) -> int: ...
	def LocateUVPoint(self, pnt: gp_Pnt2d) -> Tuple[int, int]: ...
	def LocateVParameter(self, V: float) -> int: ...
	def NbUPatches(self) -> int: ...
	def NbVPatches(self) -> int: ...
	@overload
	def Patch(self, i: int, j: int) -> Geom_Surface: ...
	@overload
	def Patch(self, U: float, V: float) -> Geom_Surface: ...
	@overload
	def Patch(self, pnt: gp_Pnt2d) -> Geom_Surface: ...
	def Patches(self) -> TColGeom_HArray2OfSurface: ...
	def SetUFirstValue(self, UFirst: float) -> None: ...
	def SetUJointValues(self, UJoints: TColStd_Array1OfReal) -> bool: ...
	def SetVFirstValue(self, VFirst: float) -> None: ...
	def SetVJointValues(self, VJoints: TColStd_Array1OfReal) -> bool: ...
	def Transform(self, T: gp_Trsf) -> None: ...
	def UGlobalToLocal(self, i: int, j: int, U: float) -> float: ...
	def UIso(self, U: float) -> Geom_Curve: ...
	def UJointValue(self, i: int) -> float: ...
	def UJointValues(self) -> TColStd_HArray1OfReal: ...
	def ULocalToGlobal(self, i: int, j: int, u: float) -> float: ...
	def UReverse(self) -> None: ...
	def UReversedParameter(self, U: float) -> float: ...
	def VGlobalToLocal(self, i: int, j: int, V: float) -> float: ...
	def VIso(self, V: float) -> Geom_Curve: ...
	def VJointValue(self, j: int) -> float: ...
	def VJointValues(self) -> TColStd_HArray1OfReal: ...
	def VLocalToGlobal(self, i: int, j: int, v: float) -> float: ...
	def VReverse(self) -> None: ...
	def VReversedParameter(self, V: float) -> float: ...
	def Value(self, pnt: gp_Pnt2d) -> gp_Pnt: ...

class ShapeExtend_Explorer:
	def __init__(self) -> None: ...
	def CompoundFromSeq(self, seqval: TopTools_HSequenceOfShape) -> TopoDS_Shape: ...
	def DispatchList(self, list: TopTools_HSequenceOfShape, vertices: TopTools_HSequenceOfShape, edges: TopTools_HSequenceOfShape, wires: TopTools_HSequenceOfShape, faces: TopTools_HSequenceOfShape, shells: TopTools_HSequenceOfShape, solids: TopTools_HSequenceOfShape, compsols: TopTools_HSequenceOfShape, compounds: TopTools_HSequenceOfShape) -> None: ...
	def ListFromSeq(self, seqval: TopTools_HSequenceOfShape, lisval: TopTools_ListOfShape, clear: Optional[bool] = True) -> None: ...
	def SeqFromCompound(self, comp: TopoDS_Shape, expcomp: bool) -> TopTools_HSequenceOfShape: ...
	def SeqFromList(self, lisval: TopTools_ListOfShape) -> TopTools_HSequenceOfShape: ...
	def ShapeType(self, shape: TopoDS_Shape, compound: bool) -> TopAbs_ShapeEnum: ...
	def SortedCompound(self, shape: TopoDS_Shape, type: TopAbs_ShapeEnum, explore: bool, compound: bool) -> TopoDS_Shape: ...

class ShapeExtend_WireData(Standard_Transient):
	@overload
	def __init__(self) -> None: ...
	@overload
	def __init__(self, wire: TopoDS_Wire, chained: Optional[bool] = True, theManifoldMode: Optional[bool] = True) -> None: ...
	@overload
	def Add(self, edge: TopoDS_Edge, atnum: Optional[int] = 0) -> None: ...
	@overload
	def Add(self, wire: TopoDS_Wire, atnum: Optional[int] = 0) -> None: ...
	@overload
	def Add(self, wire: ShapeExtend_WireData, atnum: Optional[int] = 0) -> None: ...
	@overload
	def Add(self, shape: TopoDS_Shape, atnum: Optional[int] = 0) -> None: ...
	@overload
	def AddOriented(self, edge: TopoDS_Edge, mode: int) -> None: ...
	@overload
	def AddOriented(self, wire: TopoDS_Wire, mode: int) -> None: ...
	@overload
	def AddOriented(self, shape: TopoDS_Shape, mode: int) -> None: ...
	def Clear(self) -> None: ...
	def ComputeSeams(self, enforce: Optional[bool] = True) -> None: ...
	def Edge(self, num: int) -> TopoDS_Edge: ...
	def Index(self, edge: TopoDS_Edge) -> int: ...
	@overload
	def Init(self, other: ShapeExtend_WireData) -> None: ...
	@overload
	def Init(self, wire: TopoDS_Wire, chained: Optional[bool] = True, theManifoldMode: Optional[bool] = True) -> bool: ...
	def IsSeam(self, num: int) -> bool: ...
	def GetManifoldMode(self) -> bool: ...
	def SetManifoldMode(self, value: bool) -> None: ...
	def NbEdges(self) -> int: ...
	def NbNonManifoldEdges(self) -> int: ...
	def NonmanifoldEdge(self, num: int) -> TopoDS_Edge: ...
	def NonmanifoldEdges(self) -> TopTools_HSequenceOfShape: ...
	def Remove(self, num: Optional[int] = 0) -> None: ...
	@overload
	def Reverse(self) -> None: ...
	@overload
	def Reverse(self, face: TopoDS_Face) -> None: ...
	def Set(self, edge: TopoDS_Edge, num: Optional[int] = 0) -> None: ...
	def SetDegeneratedLast(self) -> None: ...
	def SetLast(self, num: int) -> None: ...
	def Wire(self) -> TopoDS_Wire: ...
	def WireAPIMake(self) -> TopoDS_Wire: ...

class ShapeExtend_MsgRegistrator(ShapeExtend_BasicMsgRegistrator):
	def __init__(self) -> None: ...
	def MapShape(self) -> ShapeExtend_DataMapOfShapeListOfMsg: ...
	def MapTransient(self) -> ShapeExtend_DataMapOfTransientListOfMsg: ...
	@overload
	def Send(self, object: Standard_Transient, message: Message_Msg, gravity: Message_Gravity) -> None: ...
	@overload
	def Send(self, shape: TopoDS_Shape, message: Message_Msg, gravity: Message_Gravity) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

shapeextend_DecodeStatus = shapeextend.DecodeStatus
shapeextend_EncodeStatus = shapeextend.EncodeStatus
shapeextend_Init = shapeextend.Init



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _Visualization
else:
    import _Visualization

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def process_exception(error: "Standard_Failure const &", method_name: "std::string", class_name: "std::string") -> "void":
    return _Visualization.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _Visualization.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _Visualization.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Visualization.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _Visualization.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _Visualization.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _Visualization.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _Visualization.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _Visualization.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _Visualization.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _Visualization.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Visualization.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _Visualization.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _Visualization.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Visualization.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _Visualization.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _Visualization.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _Visualization.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _Visualization:
_Visualization.SwigPyIterator_swigregister(SwigPyIterator)


def Handle_AIS_InteractiveContext_Create() -> "opencascade::handle< AIS_InteractiveContext >":
    return _Visualization.Handle_AIS_InteractiveContext_Create()

def Handle_AIS_InteractiveContext_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< AIS_InteractiveContext >":
    return _Visualization.Handle_AIS_InteractiveContext_DownCast(t)

def Handle_AIS_InteractiveContext_IsNull(t: "opencascade::handle< AIS_InteractiveContext > const &") -> "bool":
    return _Visualization.Handle_AIS_InteractiveContext_IsNull(t)

def Handle_V3d_View_Create() -> "opencascade::handle< V3d_View >":
    return _Visualization.Handle_V3d_View_Create()

def Handle_V3d_View_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< V3d_View >":
    return _Visualization.Handle_V3d_View_DownCast(t)

def Handle_V3d_View_IsNull(t: "opencascade::handle< V3d_View > const &") -> "bool":
    return _Visualization.Handle_V3d_View_IsNull(t)

def Handle_V3d_Viewer_Create() -> "opencascade::handle< V3d_Viewer >":
    return _Visualization.Handle_V3d_Viewer_Create()

def Handle_V3d_Viewer_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< V3d_Viewer >":
    return _Visualization.Handle_V3d_Viewer_DownCast(t)

def Handle_V3d_Viewer_IsNull(t: "opencascade::handle< V3d_Viewer > const &") -> "bool":
    return _Visualization.Handle_V3d_Viewer_IsNull(t)
class Display3d(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(Display3d self) -> Display3d"""
        _Visualization.Display3d_swiginit(self, _Visualization.new_Display3d())
    __swig_destroy__ = _Visualization.delete_Display3d

    def Init(self, handle: "long const") -> "void":
        r"""Init(Display3d self, long const handle)"""
        return _Visualization.Display3d_Init(self, handle)

    def SetAnaglyphMode(self, mode: "int") -> "void":
        r"""SetAnaglyphMode(Display3d self, int mode)"""
        return _Visualization.Display3d_SetAnaglyphMode(self, mode)

    def SetNbMsaaSample(self, nb: "int") -> "void":
        r"""SetNbMsaaSample(Display3d self, int nb)"""
        return _Visualization.Display3d_SetNbMsaaSample(self, nb)

    def ChangeRenderingParams(self, Method: "int", RaytracingDepth: "int", IsShadowEnabled: "bool", IsReflectionEnabled: "bool", IsAntialiasingEnabled: "bool", IsTransparentShadowEnabled: "bool", StereoMode: "int", AnaglyphFilter: "int", ToReverseStere: "bool") -> "void":
        r"""ChangeRenderingParams(Display3d self, int Method, int RaytracingDepth, bool IsShadowEnabled, bool IsReflectionEnabled, bool IsAntialiasingEnabled, bool IsTransparentShadowEnabled, int StereoMode, int AnaglyphFilter, bool ToReverseStere)"""
        return _Visualization.Display3d_ChangeRenderingParams(self, Method, RaytracingDepth, IsShadowEnabled, IsReflectionEnabled, IsAntialiasingEnabled, IsTransparentShadowEnabled, StereoMode, AnaglyphFilter, ToReverseStere)

    def EnableVBO(self) -> "void":
        r"""EnableVBO(Display3d self)"""
        return _Visualization.Display3d_EnableVBO(self)

    def DisableVBO(self) -> "void":
        r"""DisableVBO(Display3d self)"""
        return _Visualization.Display3d_DisableVBO(self)

    def GetView(self) -> "Handle_V3d_View &":
        r"""GetView(Display3d self) -> Handle_V3d_View &"""
        return _Visualization.Display3d_GetView(self)

    def GetViewer(self) -> "Handle_V3d_Viewer &":
        r"""GetViewer(Display3d self) -> Handle_V3d_Viewer &"""
        return _Visualization.Display3d_GetViewer(self)

    def GetContext(self) -> "Handle_AIS_InteractiveContext":
        r"""GetContext(Display3d self) -> Handle_AIS_InteractiveContext"""
        return _Visualization.Display3d_GetContext(self)

    def Test(self) -> "void":
        r"""Test(Display3d self)"""
        return _Visualization.Display3d_Test(self)

    def InitOffscreen(self, size_x: "int", size_y: "int") -> "bool":
        r"""InitOffscreen(Display3d self, int size_x, int size_y) -> bool"""
        return _Visualization.Display3d_InitOffscreen(self, size_x, size_y)

    def SetSize(self, size_x: "int", size_y: "int") -> "bool":
        r"""SetSize(Display3d self, int size_x, int size_y) -> bool"""
        return _Visualization.Display3d_SetSize(self, size_x, size_y)

    def IsOffscreen(self) -> "bool":
        r"""IsOffscreen(Display3d self) -> bool"""
        return _Visualization.Display3d_IsOffscreen(self)

    def GetImageData(self, width: "int", height: "int", bufType: "int"=0) -> "PyObject *":
        r"""GetImageData(Display3d self, int width, int height, int bufType=0) -> PyObject"""
        return _Visualization.Display3d_GetImageData(self, width, height, bufType)

    def GetSize(self) -> "PyObject *":
        r"""GetSize(Display3d self) -> PyObject *"""
        return _Visualization.Display3d_GetSize(self)

# Register Display3d in _Visualization:
_Visualization.Display3d_swigregister(Display3d)



